! $Id: avhrr_diag_routines.f90,v 1.31 2010/09/23 16:06:00 mfoster Exp $
!-------------------------------------------------------------------------
!@***h* PATMOSX/avhrr_diag_routines.f90
! 
! NAME
!   avhrr_diag_routines.f90
! 
! LOCATION
!   /patmosx/src/avhrr_diag_routines.f90
! 
! PURPOSE
!   AVHRR DIAGNOSTIC ROUTINES
! 
! DESCRIPTION
!   this module houses the routines for performance diagnostics
!
! AUTHOR
!   Andrew Heidinger, NOAA/NESDIS
! 
! CREATION DATE
!   unk
!
! COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
!   THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
!   TO USERS.
!
! NOTES
!  Routines in this module:
!
!  OPEN_DIAG_FILES - open the files used for diagnostic output
!  WRITE_TO_DIAG_FILES - write chosen fields to diagnostic output
!  CLOSE_DIAG_FILES - close the files used for diagnostic output
!
!@*****
!-------------------------------------------------------------------------
! AVHRR DIAGNOSTIC ROUTINES
!
! this module houses the routines for performance diagnostics
!
! Author: Andrew Heidinger, NOAA/NESDIS
!
! Routines in this module:
!
! OPEN_DIAG_FILES - open the files used for diagnostic output
! WRITE_TO_DIAG_FILES - write chosen fields to diagnostic output
! CLOSE_DIAG_FILES - close the files used for diagnostic output
! 
!-------------------------------------------------------------------------
module AVHRR_DIAGNOSTIC_ROUTINES
 use CONSTANTS
 use AVHRR_PIXEL_COMMON
 use CLOUD_MASK_ROUTINES
 use FILE_UTILITY
 implicit none
 private
 public:: OPEN_DIAG_FILES,WRITE_TO_DIAG_FILES,CLOSE_DIAG_FILES

 integer(kind=int4), public, save:: j_diag_output = 0
 integer(kind=int4), parameter, private:: num_diag = 59
 integer(kind=int4), dimension(num_diag), private:: diag_lun

 INCLUDE "awg_cloud_mask_tests_thresholds.inc"

 contains

!---------------------------------------------------------------------
! subroutine OPEN_DIAG_FILES()
! Open the files used for diagnostic output
!---------------------------------------------------------------------
subroutine OPEN_DIAG_FILES()

  diag_lun(1) = get_lun()
  open(unit=diag_lun(1),file="./diag_output/pix1.dat",status="replace",form="unformatted", &
               access="direct",recl=4*num_pix,action="write")

  diag_lun(2) = get_lun()
  open(unit=diag_lun(2),file="./diag_output/pix2.dat",status="replace",form="unformatted", &
               access="direct",recl=4*num_pix,action="write")

  diag_lun(3) = get_lun()
  open(unit=diag_lun(3),file="./diag_output/lat.dat",status="replace",form="unformatted", &
               access="direct",recl=4*num_pix,action="write")

  diag_lun(4) = get_lun()
  open(unit=diag_lun(4),file="./diag_output/lon.dat",status="replace",form="unformatted", &
               access="direct",recl=4*num_pix,action="write")

  diag_lun(5) = get_lun()
  open(unit=diag_lun(5),file="./diag_output/sfc_type.dat",status="replace",form="unformatted", &
               access="direct",recl=4*num_pix,action="write")

  diag_lun(6) = get_lun()
  open(unit=diag_lun(6),file="./diag_output/alb1.dat",status="replace",form="unformatted", &
               access="direct",recl=4*num_pix,action="write")

  diag_lun(7) = get_lun()
  open(unit=diag_lun(7),file="./diag_output/alb2.dat",status="replace",form="unformatted", &
               access="direct",recl=4*num_pix,action="write")

  diag_lun(8) = get_lun()
  open(unit=diag_lun(8),file="./diag_output/alb3b.dat",status="replace",form="unformatted", &
               access="direct",recl=4*num_pix,action="write")

  diag_lun(9) = get_lun()
  open(unit=diag_lun(9),file="./diag_output/alb3a.dat",status="replace",form="unformatted", &
               access="direct",recl=4*num_pix,action="write")

  diag_lun(10) = get_lun()
  open(unit=diag_lun(10),file="./diag_output/ems3b.dat",status="replace",form="unformatted", &
               access="direct",recl=4*num_pix,action="write")

  diag_lun(11) = get_lun()
  open(unit=diag_lun(11),file="./diag_output/bt4.dat",status="replace",form="unformatted", &
               access="direct",recl=4*num_pix,action="write")

  diag_lun(12) = get_lun()
  open(unit=diag_lun(12),file="./diag_output/bt45.dat",status="replace",form="unformatted", &
               access="direct",recl=4*num_pix,action="write")

  diag_lun(13) = get_lun()
  open(unit=diag_lun(13),file="./diag_output/bt3b5.dat",status="replace",form="unformatted", &
               access="direct",recl=4*num_pix,action="write")

  diag_lun(14) = get_lun()
  open(unit=diag_lun(14),file="./diag_output/solzen.dat",status="replace",form="unformatted", &
               access="direct",recl=4*num_pix,action="write")

  diag_lun(15) = get_lun()
  open(unit=diag_lun(15),file="./diag_output/zsfc.dat",status="replace",form="unformatted", &
              access="direct",recl=4*num_pix,action="write")

  diag_lun(16) = get_lun()
  open(unit=diag_lun(16),file="./diag_output/tau_vis.dat",status="replace",form="unformatted", &
              access="direct",recl=4*num_pix,action="write")

  diag_lun(17) = get_lun()
  open(unit=diag_lun(17),file="./diag_output/re_vis.dat",status="replace",form="unformatted", &
              access="direct",recl=4*num_pix,action="write")

  diag_lun(18) = get_lun()
  open(unit=diag_lun(18),file="./diag_output/ems3b_sfc.dat",status="replace",form="unformatted", &
              access="direct",recl=4*num_pix,action="write")

  diag_lun(19) = get_lun()
  open(unit=diag_lun(19),file="./diag_output/ems3b_clr.dat",status="replace",form="unformatted", &
              access="direct",recl=4*num_pix,action="write")

  diag_lun(20) = get_lun()
  open(unit=diag_lun(20),file="./diag_output/cld_mask.dat",status="replace",form="unformatted", &
              access="direct",recl=4*num_pix,action="write")

  diag_lun(21) = get_lun()
  open(unit=diag_lun(21),file="./diag_output/rtct.dat",status="replace",form="unformatted", &
              access="direct",recl=4*num_pix,action="write")

  diag_lun(22) = get_lun()
  open(unit=diag_lun(22),file="./diag_output/etrop.dat",status="replace",form="unformatted", &
              access="direct",recl=4*num_pix,action="write")

  diag_lun(23) = get_lun()
  open(unit=diag_lun(23),file="./diag_output/pfmft.dat",status="replace",form="unformatted", &
              access="direct",recl=4*num_pix,action="write")

  diag_lun(24) = get_lun()
  open(unit=diag_lun(24),file="./diag_output/rfmft.dat",status="replace",form="unformatted", &
              access="direct",recl=4*num_pix,action="write")

  diag_lun(25) = get_lun()
  open(unit=diag_lun(25),file="./diag_output/rgct.dat",status="replace",form="unformatted", &
              access="direct",recl=4*num_pix,action="write")

  diag_lun(26) = get_lun()
  open(unit=diag_lun(26),file="./diag_output/rvct.dat",status="replace",form="unformatted", &
              access="direct",recl=4*num_pix,action="write")

  diag_lun(27) = get_lun()
  open(unit=diag_lun(27),file="./diag_output/nirref.dat",status="replace",form="unformatted", &
              access="direct",recl=4*num_pix,action="write")

  diag_lun(28) = get_lun()
  open(unit=diag_lun(28),file="./diag_output/emiss4.dat",status="replace",form="unformatted", &
              access="direct",recl=4*num_pix,action="write")

  diag_lun(29) = get_lun()
  open(unit=diag_lun(29),file="./diag_output/ulst.dat",status="replace",form="unformatted", &
              access="direct",recl=4*num_pix,action="write")

  diag_lun(30) = get_lun()
  open(unit=diag_lun(30),file="./diag_output/tc_sw.dat",status="replace",form="unformatted", &
              access="direct",recl=4*num_pix,action="write")

  diag_lun(31) = get_lun()
  open(unit=diag_lun(31),file="./diag_output/ec_sw.dat",status="replace",form="unformatted", &
              access="direct",recl=4*num_pix,action="write")

  diag_lun(32) = get_lun()
  open(unit=diag_lun(32),file="./diag_output/beta_sw.dat",status="replace",form="unformatted", &
              access="direct",recl=4*num_pix,action="write")

  diag_lun(33) = get_lun()
  open(unit=diag_lun(33),file="./diag_output/bt4_rtm.dat",status="replace",form="unformatted", &
              access="direct",recl=4*num_pix,action="write")

  diag_lun(34) = get_lun()
  open(unit=diag_lun(34),file="./diag_output/bt5_rtm.dat",status="replace",form="unformatted", &
              access="direct",recl=4*num_pix,action="write")

  diag_lun(35) = get_lun()
  open(unit=diag_lun(35),file="./diag_output/ec_tropo.dat",status="replace",form="unformatted", &
              access="direct",recl=4*num_pix,action="write")

  diag_lun(36) = get_lun()
  open(unit=diag_lun(36),file="./diag_output/snow.dat",status="replace",form="unformatted", &
              access="direct",recl=4*num_pix,action="write")

  diag_lun(37) = get_lun()
  open(unit=diag_lun(37),file="./diag_output/smoke.dat",status="replace",form="unformatted", &
              access="direct",recl=4*num_pix,action="write")

  diag_lun(38) = get_lun()
  open(unit=diag_lun(38),file="./diag_output/nfmft.dat",status="replace",form="unformatted", &
              access="direct",recl=4*num_pix,action="write")

  diag_lun(39) = get_lun()
  open(unit=diag_lun(39),file="./diag_output/ec_tropo_nadir.dat",status="replace",form="unformatted", &
              access="direct",recl=4*num_pix,action="write")

  diag_lun(40) = get_lun()
  open(unit=diag_lun(40),file="./diag_output/cld_mask_bayes.dat",status="replace",form="unformatted", &
              access="direct",recl=4*num_pix,action="write")

  diag_lun(41) = get_lun()
  open(unit=diag_lun(41),file="./diag_output/post_prob.dat",status="replace",form="unformatted", &
               access="direct",recl=4*num_pix,action="write")

  diag_lun(42) = get_lun()
  open(unit=diag_lun(42),file="./diag_output/cld_type.dat",status="replace",form="unformatted", &
               access="direct",recl=4*num_pix,action="write")

  diag_lun(43) = get_lun()
  open(unit=diag_lun(43),file="./diag_output/land.dat",status="replace",form="unformatted", &
               access="direct",recl=4*num_pix,action="write")

  diag_lun(44) = get_lun()
  open(unit=diag_lun(44),file="./diag_output/tsfc_rtm.dat",status="replace",form="unformatted", &
               access="direct",recl=4*num_pix,action="write")

  diag_lun(45) = get_lun()
  open(unit=diag_lun(45),file="./diag_output/pc_sw.dat",status="replace",form="unformatted", &
               access="direct",recl=4*num_pix,action="write")

  diag_lun(46) = get_lun()
  open(unit=diag_lun(46),file="./diag_output/dust.dat",status="replace",form="unformatted", &
               access="direct",recl=4*num_pix,action="write")

  diag_lun(47) = get_lun()
  open(unit=diag_lun(47),file="./diag_output/smoke.dat",status="replace",form="unformatted", &
               access="direct",recl=4*num_pix,action="write")

  diag_lun(48) = get_lun()
  open(unit=diag_lun(48),file="./diag_output/bad_pixel_mask.dat",status="replace",form="unformatted", &
               access="direct",recl=4*num_pix,action="write")

  diag_lun(49) = get_lun()
  open(unit=diag_lun(49),file="./diag_output/bt4_std_3x3.dat",status="replace",form="unformatted", &
               access="direct",recl=4*num_pix,action="write")

  diag_lun(50) = get_lun()
  open(unit=diag_lun(50),file="./diag_output/bad_pixel_mask.dat",status="replace",form="unformatted", &
               access="direct",recl=4*num_pix,action="write")
  diag_lun(51) = get_lun()
  open(unit=diag_lun(51),file="./diag_output/alb1_sfc_white_sky.dat",status="replace",form="unformatted", &
               access="direct",recl=4*num_pix,action="write")
  diag_lun(52) = get_lun()   
  open(unit=diag_lun(52),file="./diag_output/alb3a_sfc_white_sky.dat",status="replace",form="unformatted", &
               access="direct",recl=4*num_pix,action="write")
  diag_lun(53) = get_lun()
  open(unit=diag_lun(53),file="./diag_output/cost_dcomp.dat",status="replace",form="unformatted", &
               access="direct",recl=4*num_pix,action="write")
  diag_lun(54) = get_lun()
  open(unit=diag_lun(54),file="./diag_output/error_cov_matrix_cod.dat",status="replace",form="unformatted", &
               access="direct",recl=4*num_pix,action="write")
  diag_lun(55) = get_lun()
  open(unit=diag_lun(55),file="./diag_output/error_cov_matrix_ref.dat",status="replace",form="unformatted", &
               access="direct",recl=4*num_pix,action="write")
  diag_lun(56) = get_lun()
  open(unit=diag_lun(56),file="./diag_output/zc_sw.dat",status="replace",form="unformatted", &
               access="direct",recl=4*num_pix,action="write")
  diag_lun(57) = get_lun()
  open(unit=diag_lun(57),file="./diag_output/glintzen.dat",status="replace",form="unformatted", &
               access="direct",recl=4*num_pix,action="write")
  diag_lun(58) = get_lun()
  open(unit=diag_lun(58),file="./diag_output/scatzen.dat",status="replace",form="unformatted", &
               access="direct",recl=4*num_pix,action="write")
  diag_lun(59) = get_lun()
  open(unit=diag_lun(59),file="./diag_output/glint_mask.dat",status="replace",form="unformatted", &
               access="direct",recl=4*num_pix,action="write")

 end subroutine OPEN_DIAG_FILES
!---------------------------------------------------------------------
! subroutine WRITE_TO_DIAG_FILES()
! write data to the files used for diagnostic output
!---------------------------------------------------------------------
 subroutine WRITE_TO_DIAG_FILES(j)

      integer:: j

!---------- diagnostic output
      j_diag_output = j_diag_output + 1
      write(unit=diag_lun(1),rec=j_diag_output+1) pix_array_1(:,j)
      write(unit=diag_lun(2),rec=j_diag_output+1) pix_array_2(:,j)
      write(unit=diag_lun(3),rec=j_diag_output+1) lat(:,j)
      write(unit=diag_lun(4),rec=j_diag_output+1) lon(:,j)
      write(unit=diag_lun(5),rec=j_diag_output+1) real(sfc_type(:,j))
      write(unit=diag_lun(6),rec=j_diag_output+1) alb1(:,j)
      write(unit=diag_lun(7),rec=j_diag_output+1) alb2(:,j)
      write(unit=diag_lun(8),rec=j_diag_output+1) alb3b(:,j)
      write(unit=diag_lun(9),rec=j_diag_output+1) alb3a(:,j)
      write(unit=diag_lun(10),rec=j_diag_output+1) ems3b(:,j)
      write(unit=diag_lun(11),rec=j_diag_output+1) bt4(:,j)
      write(unit=diag_lun(12),rec=j_diag_output+1) bt45(:,j)
      write(unit=diag_lun(13),rec=j_diag_output+1) bt3b5(:,j)
      write(unit=diag_lun(14),rec=j_diag_output+1) solzen(:,j)
      write(unit=diag_lun(15),rec=j_diag_output+1) zsfc(:,j)
      write(unit=diag_lun(16),rec=j_diag_output+1) tau_vis(:,j)
      write(unit=diag_lun(17),rec=j_diag_output+1) re_vis(:,j)
      write(unit=diag_lun(18),rec=j_diag_output+1) ems3b_sfc(:,j)
      write(unit=diag_lun(19),rec=j_diag_output+1) ems3b_clear_rtm(:,j)

      write(unit=diag_lun(20),rec=j_diag_output+1) real(cld_mask(:,j))

      !--- cloud mask tests
      write(unit=diag_lun(21),rec=j_diag_output+1)  &
             real(cld_test_vector(First_IR_Cld_Mask_Test + 0,:,j))

      write(unit=diag_lun(22),rec=j_diag_output+1)  &
             real(cld_test_vector(First_IR_Cld_Mask_Test + 1,:,j))

      write(unit=diag_lun(23),rec=j_diag_output+1)  &
             real(cld_test_vector(First_IR_Cld_Mask_Test + 2,:,j))

      write(unit=diag_lun(24),rec=j_diag_output+1)  &
             real(cld_test_vector(First_IR_Cld_Mask_Test + 4,:,j))

      write(unit=diag_lun(25),rec=j_diag_output+1)  &
             real(cld_test_vector(First_VIS_Cld_Mask_Test + 0,:,j))

      write(unit=diag_lun(26),rec=j_diag_output+1)  &
             real(cld_test_vector(First_VIS_Cld_Mask_Test + 1,:,j))

      write(unit=diag_lun(27),rec=j_diag_output+1)  &
             real(cld_test_vector(First_SWIR_Solar_Cld_Mask_Test + 0,:,j))

      write(unit=diag_lun(28),rec=j_diag_output+1)  &
             real(cld_test_vector(First_SWIR_Thermal_Cld_Mask_Test + 0,:,j))

      write(unit=diag_lun(29),rec=j_diag_output+1)  &
             real(cld_test_vector(First_SWIR_Thermal_Cld_Mask_Test + 1,:,j))

      !--- split window cloud heights
      write(unit=diag_lun(30),rec=j_diag_output+1) tc_sw(:,j)
      write(unit=diag_lun(31),rec=j_diag_output+1) ec_sw(:,j)
      write(unit=diag_lun(32),rec=j_diag_output+1) beta_sw(:,j)

      write(unit=diag_lun(33),rec=j_diag_output+1) bt_clear_ch4_rtm(:,j)
      write(unit=diag_lun(34),rec=j_diag_output+1) bt_clear_ch5_rtm(:,j)
      write(unit=diag_lun(35),rec=j_diag_output+1) emiss_11_tropo_rtm(:,j)
      write(unit=diag_lun(36),rec=j_diag_output+1) real(snow(:,j))
      write(unit=diag_lun(37),rec=j_diag_output+1) real(smoke(:,j))
      write(unit=diag_lun(38),rec=j_diag_output+1)  &
             real(cld_test_vector(First_IR_Cld_Mask_Test + 3,:,j))
      write(unit=diag_lun(39),rec=j_diag_output+1) emiss_11_tropo_nadir_rtm(:,j)
      write(unit=diag_lun(40),rec=j_diag_output+1) real(cld_mask_bayes(:,j))
      write(unit=diag_lun(41),rec=j_diag_output+1) posterior_cld_probability(:,j)
      write(unit=diag_lun(42),rec=j_diag_output+1) real(cld_type(:,j))
      write(unit=diag_lun(43),rec=j_diag_output+1) real(land(:,j))
      write(unit=diag_lun(44),rec=j_diag_output+1) tsfc_rtm(:,j)
      write(unit=diag_lun(45),rec=j_diag_output+1) pc_sw(:,j)
      write(unit=diag_lun(46),rec=j_diag_output+1) real(dust(:,j))
      write(unit=diag_lun(47),rec=j_diag_output+1) real(smoke(:,j))
      write(unit=diag_lun(48),rec=j_diag_output+1) real(bad_pixel_mask(:,j))
      write(unit=diag_lun(49),rec=j_diag_output+1) bt4_std_3x3(:,j)
      write(unit=diag_lun(50),rec=j_diag_output+1) real(bad_pixel_mask(:,j))
      write(unit=diag_lun(51),rec=j_diag_output+1) real(alb1_sfc_white_sky(:,j))
      write(unit=diag_lun(52),rec=j_diag_output+1) real(alb3a_sfc_white_sky(:,j))
      write(unit=diag_lun(53),rec=j_diag_output+1) real(cost_dcomp(:,j))
      write(unit=diag_lun(54),rec=j_diag_output+1) real(error_cov_matrix_cod(:,j))
      write(unit=diag_lun(55),rec=j_diag_output+1) real(error_cov_matrix_ref(:,j))
      write(unit=diag_lun(56),rec=j_diag_output+1) zc_sw(:,j)
      write(unit=diag_lun(57),rec=j_diag_output+1) glintzen(:,j)
      write(unit=diag_lun(58),rec=j_diag_output+1) scatangle(:,j)
      write(unit=diag_lun(59),rec=j_diag_output+1) real(glint_mask(:,j))
 end subroutine WRITE_TO_DIAG_FILES


!---------------------------------------------------------------------
! subroutine CLOSE_DIAG_FILES()
! Close the files used for diagnostic output
!---------------------------------------------------------------------
 subroutine CLOSE_DIAG_FILES()

  integer:: idiag

     do idiag = 1, num_diag
       write(unit=diag_lun(idiag),rec=1) num_pix, j_diag_output, asc_flag_diag, lat_max_diag, lat_min_diag, file_1b
       close(unit=diag_lun(idiag))
     end do

 end subroutine CLOSE_DIAG_FILES

!-----------------------------------------------------------
! end of module
!-----------------------------------------------------------
end module AVHRR_DIAGNOSTIC_ROUTINES
