! $Id: avhrr_pixel_common.f90,v 1.65 2010/08/29 00:59:03 heidinger Exp $
!-------------------------------------------------------------------------
!@***h* PATMOSX/avhrr_pixel_common.f90
! 
! NAME
!   avhrr_pixel_common.f90
! 
! LOCATION
!   /patmosx/src/avhrr_pixel_common.f90
! 
! PURPOSE
!   AVHRR PIXEL COMMON
! 
! DESCRIPTION
!   This module houses routines used to handle the pixel level arrays that
!   are based through this module
!
! AUTHOR
!   Andrew Heidinger, NOAA/NESDIS
! 
! CREATION DATE
!   unk
!
! COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
!   THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
!   TO USERS.
!
! NOTES
!
!
! EXTERNALS
!  Public routines in this module:
!   CREATE_PIXEL_ARRAYS - allocate memory for pixel level arrays
!   DESTROY_PIXEL_ARRAYS - deallocate memory for pixel level arrays
!   RESET_PIXEL_ARRAYS_TO_MISSING - set pixel arrays to missing
!
!@*****
! AVHRR PIXEL COMMON
!
! This module houses routines used to handle the pixel level arrays that
! are based through this module
!
! Author: Andrew Heidinger, NOAA/NESDIS
!
! Public routines in this module:
!
! CREATE_PIXEL_ARRAYS - allocate memory for pixel level arrays
! DESTROY_PIXEL_ARRAYS - deallocate memory for pixel level arrays
! RESET_PIXEL_ARRAYS_TO_MISSING - set pixel arrays to missing
!
! File I/O: None
!   
!-------------------------------------------------------------------------
module AVHRR_PIXEL_COMMON
 use CONSTANTS
 implicit none
 private
 public:: CREATE_PIXEL_ARRAYS, &
          DESTROY_PIXEL_ARRAYS, &
          RESET_PIXEL_ARRAYS_TO_MISSING

!--- variables read in from clavrxorb_defaults_options file
  integer,public, save:: gac
  integer,public, save:: klm
  integer,public, save:: aapp
  integer,public, save:: avhrr_1_flag
  integer,public, save:: bx_file_flag
  integer,public, save:: cmr_file_flag
  integer,public, save:: cloud_mask_1b
  integer,public, save:: cloud_mask_bayesian_flag
  integer,public, save:: cloud_phase_bayesian_flag
  integer,public, save:: level3_flag
  integer,public, save:: cloud_type_ir_flag
  integer,public, save:: ref_cal_1b 
  integer,public, save:: therm_cal_1b
  integer,public, save:: nav_flag       !0=level1b,1=clevernav,2=reposnx
  integer,public, save:: nav_file_flag  !yes, write out a navigation file
  integer,public, save:: obs_file_flag
  integer,public, save:: geo_file_flag
  integer,public, save:: sst_file_flag
  integer,public, save:: cld_file_flag
  integer,public, save:: rtm_file_flag
  integer,public, save:: ash_file_flag
  integer,public, save:: level2_file_flag
  integer,public, save:: subset_pixel_hdf_flag
  integer,public, save:: use_sst_anal
  integer,public, save:: use_sst_anal_default
  integer,public, save:: l1b_gzip
  integer,public, save:: l1b_bzip2
  integer,public, save:: diag_flag
  integer,public, save:: sst_anal_opt
  integer,public, save:: data_comp_flag
  integer,public, save:: use_seebor
  integer,public, save:: read_volcano_mask
  integer,public, save:: read_land_mask
  integer,public, save:: read_coast_mask
  integer,public, save:: read_surface_elevation
  integer,public, save:: read_hires_sfc_type
  integer,public, save:: read_snow_mask
  integer,public, save:: machine_byte_ordering
  integer,public, save:: lrc_flag  !local radiative center flag
  integer,public, save:: process_undetected_cloud_flag
  integer,public, save:: failed_hires_snow_mask_flag
  integer,public, save:: output_scaled_reflectances
  integer,public, save:: ncdc_level2_flag

  integer(kind=int4), public, save:: byte_swap_1b
  integer(kind=int4), public, save:: lun_level1b

  integer,public, save:: cld_flag
  integer,public, save:: sst_flag
  integer,public, save:: aer_flag
  integer,public, save:: ash_flag
  integer,public, save:: erb_flag
  integer,public, save:: nwp_flag
  integer,public, save:: modis_clr_alb_flag
  integer,public, save:: asc_flag_diag
  integer,public, save:: rtm_flag
  integer,public, save:: prob_clear_res_flag
  integer,public, save:: smooth_nwp

  character(len=128),public,save::file_1b
  character(len=128),public,save::file_1bx
  character(len=128),public,save::file_cmr
  character(len=128),public,save::file_sst
  character(len=128),public,save:: ancil_data_dir
  character(len=128),public,save:: gfs_data_dir
  character(len=128),public,save:: ncep_data_dir
  character(len=128),public,save:: oisst_data_dir
  character(len=128),public,save:: snow_data_dir
  character(len=128),public,save:: dir_1b
  character(len=128),public,save:: dir_1bx
  character(len=128),public,save:: file_nav
  character(len=128),public,save:: instr_const_file
  character(len=128),public,save:: algo_const_file
  character(len=128),public,save:: dir_cmr
  character(len=128),public,save:: dir_sst
  character(len=128),public,save:: dir_level3
  character(len=128),public,save:: dir_nav_in
  character(len=128),public,save:: dir_nav_out
  character(len=128),public,save:: dir_cld
  character(len=128),public,save:: dir_obs
  character(len=128),public,save:: dir_geo
  character(len=128),public,save:: dir_rtm
  character(len=128),public,save:: dir_ash
  character(len=128),public,save:: dir_level2

 real(kind=real4), public, save:: dlat
 real(kind=real4), public, save:: lat_min_diag
 real(kind=real4), public, save:: lat_max_diag
 real(kind=real4), public, save:: solzen_min_limit
 real(kind=real4), public, save:: solzen_max_limit
 real(kind=real4), public, save:: timerr_seconds

 integer, public, save:: level3_format
 real(kind=real4), public, save:: dlat_hist2d

 !----- variables pertaining to scanline size
  integer, public, save:: num_scans_per_segment
  integer, public, save:: num_scans_read
  integer, public, save:: num_segments
  integer, public, save:: j_min
  integer, public, save:: j_max
  integer, public, save:: ilast_read
  integer, public, save:: ifirst_read
  integer(kind=int4), public, save:: num_pix
  integer(kind=int4), public, save:: l1b_rec_length
  integer(kind=int4), public, save:: num_anchors

  !------- pixel array declarations
  integer (kind=int1), dimension(:), allocatable, public:: header_buffer
  integer (kind=int1), dimension(:), allocatable, public:: buffer
  integer (kind=int1), dimension(:), allocatable, public:: segment_buffer
  integer (kind=int1), dimension(:), allocatable, public:: fatal
  integer (kind=int1), dimension(:), allocatable, public:: ascend
  integer (kind=int1), dimension(:), allocatable, public:: bad_scan_flag
  integer (kind=int1), dimension(:), allocatable, public:: ch3a_on
  integer (kind=int1), dimension(:), allocatable, public:: resync
  integer (kind=int1), dimension(:), allocatable, public:: clavr_status_1b
  integer (kind=int1), dimension(:), allocatable, public:: clavr_status
  integer (kind=int4), dimension(:), allocatable, public:: scan_number
  integer (kind=int4), dimension(:), allocatable, public:: scan_time
  integer (kind=int2), dimension(:), allocatable, public:: scan_day
  integer (kind=int2), dimension(:), allocatable, public:: scan_year
  real (kind=real4), dimension(:), allocatable, public:: utc_scan_time_hours
  real (kind=real4), dimension(:,:), allocatable, public:: pixel_local_time_hours
  real (kind=real4), dimension(:,:), allocatable, public:: pixel_time

  real (kind=real4), dimension(:,:), allocatable,save,public:: solzen_anchor
  real (kind=real4), dimension(:,:), allocatable,save,public:: lat_anchor_1b
  real (kind=real4), dimension(:,:), allocatable,save,public:: lon_anchor_1b
  real (kind=real4), dimension(:,:), allocatable,save,public:: satzen_anchor
  real (kind=real4), dimension(:,:), allocatable,save,public:: relaz_anchor
  real (kind=real4), dimension(:,:), allocatable,save,public:: solaz_anchor
  real (kind=real4), dimension(:,:), allocatable,save,public:: glintzen_anchor
  real (kind=real4), dimension(:,:), allocatable,save,public:: scatangle_anchor

!--- parameters from avhrr header
  integer(kind=int4), public, save:: num_scans, num_scans_hdf
  integer(kind=int2), public, save:: sc_id,ver_1b,data_type, num_loc, tip_parity, aux_sync,  &
                                     ramp_auto_cal,start_year_prev, start_day_prev, &
                                     month, month_prev,day_of_month,ileap
  integer(kind=int4), public, save:: sc_id_new, sc_id_new_prev
  character(len=6), public, save:: sc_id_char
  character(len=7),public,save:: proc_block_id
  integer(kind=int4), save, public:: orbit_number
  integer(kind=int4), save, public:: start_time
  integer(kind=int4), save, public:: end_time
  integer(kind=int2), save, public:: start_year
  integer(kind=int2), save, public:: end_year
  integer(kind=int2), save, public:: start_day
  integer(kind=int2), save, public:: end_day



  !--- instrument counts
  integer (kind=int2), dimension(:,:,:), allocatable, public,save:: chan_counts
  real(kind=int4), dimension(:,:), allocatable, public, save:: scan_space_counts
  real(kind=int4), dimension(:,:), allocatable, public, save:: scan_bb_counts
  integer (kind=int2), dimension(:,:,:), allocatable, public,save:: chan_counts_sg


  !--- calibration coefficients
  real (kind=real4), dimension(:,:), allocatable, public,save:: ir_coef_1_1b,ir_coef_2_1b,ir_coef_3_1b
  real (kind=real4), dimension(:,:), allocatable, public,save:: ir_linear_slope_1b
  real (kind=real4), dimension(:,:), allocatable, public,save:: ir_linear_intercept_1b
  real (kind=real4), dimension(:,:), allocatable, public,save:: ir_linear_slope_new
  real (kind=real4), dimension(:,:), allocatable, public,save:: ir_linear_intercept_new
  real (kind=real4), dimension(:,:), allocatable, public,save:: vis_slope_1
  real (kind=real4), dimension(:,:), allocatable, public,save:: vis_slope_2
  real (kind=real4), dimension(:,:), allocatable, public,save:: vis_intercept_1
  real (kind=real4), dimension(:,:), allocatable, public,save:: vis_intercept_2
  real (kind=real4), dimension(:,:), allocatable, public,save:: vis_intersection

  !--- calibrated observations
  real (kind=real4), dimension(:,:), allocatable, public, save:: alb1
  real (kind=real4), dimension(:,:), allocatable, public, save:: alb2
  real (kind=real4), dimension(:,:), allocatable, public, save:: alb3a
  real (kind=real4), dimension(:,:), allocatable, public, save:: alb3b
  real (kind=real4), dimension(:,:), allocatable, public, save:: rad3b
  real (kind=real4), dimension(:,:), allocatable, public, save:: rad4
  real (kind=real4), dimension(:,:), allocatable, public, save:: rad5
  real (kind=real4), dimension(:,:), allocatable, public, save:: rad3b_ems
  real (kind=real4), dimension(:,:), allocatable, public, save:: bt3b
  real (kind=real4), dimension(:,:), allocatable, public, save:: bt4
  real (kind=real4), dimension(:,:), allocatable, public, save:: bt5
  real (kind=real4), dimension(:,:), allocatable, public, save:: solzen
  real (kind=real4), dimension(:,:), allocatable, public, save:: lat
  real (kind=real4), dimension(:,:), allocatable, public, save:: lon
  real (kind=real4), dimension(:,:), allocatable, public, save:: lat_1b
  real (kind=real4), dimension(:,:), allocatable, public, save:: lon_1b
  real (kind=real4), dimension(:,:), allocatable, public, save:: glintzen
  real (kind=real4), dimension(:,:), allocatable, public, save:: satzen
  real (kind=real4), dimension(:,:), allocatable, public, save:: relaz
  real (kind=real4), dimension(:,:), allocatable, public, save:: solaz
  real (kind=real4), dimension(:,:), allocatable, public, save:: seczen
  real (kind=real4), dimension(:,:), allocatable, public, save:: ems3b
  real (kind=real4), dimension(:,:), allocatable, public, save:: zsfc
  real (kind=real4), dimension(:,:), allocatable, public, save:: coszen
  real (kind=real4), dimension(:,:), allocatable, public, save:: cossolzen
  real (kind=real4), dimension(:,:), allocatable, public, save:: scatangle
  real (kind=real4), dimension(:,:), allocatable, public, save:: airmass
  real (kind=real4), dimension(:,:), allocatable, public, save:: sst_anal
  real (kind=real4), dimension(:,:), allocatable, public, save:: sst_anal_err
  real (kind=real4), dimension(:,:), allocatable, public, save:: sst_anal_uni
  real (kind=real4), dimension(:,:), allocatable, public, save:: sst_anal_cice
  real (kind=real4), dimension(:,:), allocatable, public, save:: tsfc
  real (kind=real4), dimension(:,:), allocatable, public, save:: temp_pix_array
  real (kind=real4), dimension(:,:), allocatable, public, save:: temp_pix_array_2
  real (kind=real4), dimension(:,:), allocatable, public, save:: pix_array_1
  real (kind=real4), dimension(:,:), allocatable, public, save:: pix_array_2


  !uniformity metrics
  real(kind=real4), dimension(:,:), allocatable, public, save:: bt4_mean_3x3
  real(kind=real4), dimension(:,:), allocatable, public, save:: bt4_min_3x3
  real(kind=real4), dimension(:,:), allocatable, public, save:: bt4_max_3x3
  real(kind=real4), dimension(:,:), allocatable, public, save:: bt4_std_3x3
  real(kind=real4), dimension(:,:), allocatable, public, save:: alb1_mean_3x3
  real(kind=real4), dimension(:,:), allocatable, public, save:: alb1_min_3x3
  real(kind=real4), dimension(:,:), allocatable, public, save:: alb1_max_3x3
  real(kind=real4), dimension(:,:), allocatable, public, save:: alb1_std_3x3
  real(kind=real4), dimension(:,:), allocatable, public, save:: bt3b_mean_3x3
  real(kind=real4), dimension(:,:), allocatable, public, save:: bt3b_min_3x3
  real(kind=real4), dimension(:,:), allocatable, public, save:: bt3b_max_3x3
  real(kind=real4), dimension(:,:), allocatable, public, save:: bt3b_std_3x3
  real(kind=real4), dimension(:,:), allocatable, public, save:: bt45_mean_3x3
  real(kind=real4), dimension(:,:), allocatable, public, save:: bt45_min_3x3
  real(kind=real4), dimension(:,:), allocatable, public, save:: bt45_max_3x3
  real(kind=real4), dimension(:,:), allocatable, public, save:: bt45_std_3x3
  real(kind=real4), dimension(:,:), allocatable, public, save:: ems3b_mean_3x3
  real(kind=real4), dimension(:,:), allocatable, public, save:: ems3b_min_3x3
  real(kind=real4), dimension(:,:), allocatable, public, save:: ems3b_max_3x3
  real(kind=real4), dimension(:,:), allocatable, public, save:: ems3b_std_3x3
  real(kind=real4), dimension(:,:), allocatable, public, save:: zsfc_mean_3x3
  real(kind=real4), dimension(:,:), allocatable, public, save:: zsfc_min_3x3
  real(kind=real4), dimension(:,:), allocatable, public, save:: zsfc_max_3x3
  real(kind=real4), dimension(:,:), allocatable, public, save:: zsfc_std_3x3
  real(kind=real4), dimension(:,:), allocatable, public, save:: ems3b_median_3x3
  real(kind=real4), dimension(:,:), allocatable, public, save:: ems3b_std_median_3x3
  real(kind=real4), dimension(:,:), allocatable, public, save:: bt3b_median_3x3
  real(kind=real4), dimension(:,:), allocatable, public, save:: bt3b_std_median_3x3
  real(kind=real4), dimension(:,:), allocatable, public, save:: bt4_median_3x3
  real(kind=real4), dimension(:,:), allocatable, public, save:: bt4_std_median_3x3
  real(kind=real4), dimension(:,:), allocatable, public, save:: alb1_sfc_white_sky_mean_3x3
  real(kind=real4), dimension(:,:), allocatable, public, save:: alb1_sfc_white_sky_max_3x3
  real(kind=real4), dimension(:,:), allocatable, public, save:: alb1_sfc_white_sky_min_3x3
  real(kind=real4), dimension(:,:), allocatable, public, save:: alb1_sfc_white_sky_std_3x3
  real(kind=real4), dimension(:,:), allocatable, public, save:: bt45_bt4_max_3x3
  real(kind=real4), dimension(:,:), allocatable, public, save:: cloud_fraction_3x3
  real(kind=real4), dimension(:,:), allocatable, public, save:: cloud_fraction_uncer_3x3

  integer(kind=int4), dimension(:,:), allocatable, public, save:: i_max_bt4_3x3
  integer(kind=int4), dimension(:,:), allocatable, public, save:: j_max_bt4_3x3
  integer(kind=int4), dimension(:,:), allocatable, public, save:: i_min_bt4_3x3
  integer(kind=int4), dimension(:,:), allocatable, public, save:: j_min_bt4_3x3

  integer (kind=int1), dimension(:,:), allocatable, public:: dcc_mask !deep conv cld mask

  real (kind=real4), dimension(:,:), allocatable, public:: sst_mask   !sst used in cld mask
  real (kind=real4), dimension(:,:), allocatable, public:: sst_oper   !sst using oper. eqns
  real (kind=real4), dimension(:,:), allocatable, public:: sst_masked !sst where non-clear ocean is masked
  real (kind=real4), dimension(:,:), allocatable, public:: ndvi
  real (kind=real4), dimension(:,:), allocatable, public:: ndsi
  real (kind=real4), dimension(:,:), allocatable, public:: bt45
  real (kind=real4), dimension(:,:), allocatable, public:: bt3b4
  real (kind=real4), dimension(:,:), allocatable, public:: bt3b5
  real (kind=real4), dimension(:,:), allocatable, public:: alb21
  real (kind=real4), dimension(:,:), allocatable, public:: alb31

  integer(kind=int1), dimension(:,:), allocatable, public:: land
  integer(kind=int1), dimension(:,:), allocatable, public:: coast
  integer(kind=int1), dimension(:,:), allocatable, public:: desert_mask
  integer(kind=int1), dimension(:,:), allocatable, public:: snow_hires
  integer(kind=int1), dimension(:,:), allocatable, public:: sfc_type
  integer(kind=int1), dimension(:,:), allocatable, public:: snow
  integer(kind=int1), dimension(:,:), allocatable, public:: dust
  integer(kind=int1), dimension(:,:), allocatable, public:: smoke
  integer(kind=int1), dimension(:,:), allocatable, public:: fire
  integer(kind=int1), dimension(:,:), allocatable, public:: solar_contamination_mask
  integer(kind=int1), dimension(:,:), allocatable, public:: bad_pixel_mask
  integer(kind=int1), dimension(:,:), allocatable, public:: ch3a_on_pixel_mask
  integer(kind=int1), dimension(:,:), allocatable, public:: volcano_mask
  integer(kind=int1), dimension(:,:), allocatable, public:: coast_mask_nwp
  integer(kind=int1), dimension(:,:), allocatable, public:: space_mask

!--- mask arrays
  integer(kind=int1), dimension(:,:), allocatable, public:: land_mask
  integer(kind=int1), dimension(:,:), allocatable, public:: coast_mask 
  integer(kind=int1), dimension(:,:), allocatable, public:: glint_mask
  integer(kind=int1), dimension(:,:), allocatable, public:: day_mask 
  integer(kind=int1), dimension(:,:), allocatable, public:: rut
  integer(kind=int1), dimension(:,:), allocatable, public:: tut 

!--- pixel level surface emissivity
  real (kind=real4), dimension(:,:), allocatable, public:: sfc_emiss_3b
  real (kind=real4), dimension(:,:), allocatable, public:: sfc_emiss_4
  real (kind=real4), dimension(:,:), allocatable, public:: sfc_emiss_5

!--- cloud mask arrays
  integer (kind=int1), dimension(:,:,:), allocatable, public, save:: cld_test_vector
  integer (kind=int1), dimension(:,:,:), allocatable, public, save:: cld_test_vector_packed
  integer (kind=int1),dimension(:,:),allocatable, public, save:: cld_mask_1b
  integer (kind=int1),dimension(:,:),allocatable, public, save:: cld_mask
  integer (kind=int1),dimension(:,:),allocatable, public, save:: cld_mask_ir
  integer (kind=int1),dimension(:,:),allocatable, public, save:: cld_mask_qf
  integer (kind=int1),dimension(:,:),allocatable, public, save:: cld_mask_bayes
  integer (kind=int1),dimension(:,:),allocatable, public, save:: cld_phase_bayes
  real (kind=int4),dimension(:,:),allocatable, public, save:: &
                                                       posterior_cld_probability
  real (kind=int4),dimension(:,:),allocatable, public, save:: &
                                                       posterior_phase_probability
  integer (kind=int1),dimension(:,:),allocatable, public, save:: cld_type
  integer (kind=int1),dimension(:,:),allocatable, public, save:: cld_type_inoue
  integer (kind=int1),dimension(:,:),allocatable, public, save:: data_bit
  integer (kind=int1),dimension(:,:),allocatable, public, save, target:: temp_mask
  integer (kind=int1),dimension(:,:),allocatable, public, save:: cirrus_quality
  integer (kind=int1),dimension(:,:),allocatable, public, save:: cld_type_ir
  integer (kind=int1),dimension(:,:),allocatable, public, save:: cld_type_bayes


!--- pixel level cloud props

!-- split-window cloud algorithm results
     real (kind=real4), dimension(:,:), allocatable, public, save:: Tc_sw
     real (kind=real4), dimension(:,:), allocatable, public, save:: ec_sw
     real (kind=real4), dimension(:,:), allocatable, public, save:: Pc_sw
     real (kind=real4), dimension(:,:), allocatable, public, save:: Zc_sw
     real (kind=real4), dimension(:,:), allocatable, public, save:: beta_sw
     real (kind=real4), dimension(:,:), allocatable, public, save:: tau_sw
     real (kind=real4), dimension(:,:), allocatable, public, save:: Tc_sw_error
     real (kind=real4), dimension(:,:), allocatable, public, save:: ec_sw_error
     real (kind=real4), dimension(:,:), allocatable, public, save:: beta_sw_error
     integer (kind=int1), dimension(:,:), allocatable, public, save:: Tc_sw_qf
     integer (kind=int1), dimension(:,:), allocatable, public, save:: ec_sw_qf
     integer (kind=int1), dimension(:,:), allocatable, public, save:: cld_layer_sw
     integer (kind=int1), dimension(:,:), allocatable, public, save:: beta_sw_qf

!-- vis cloud algorithm results
     real (kind=real4), dimension(:,:), allocatable, public,target, save:: tau_vis
     real (kind=real4), dimension(:,:), allocatable, public,target, save:: tau_vis_ap
     real (kind=real4), dimension(:,:), allocatable, public,target, save:: vis_ref_fm
     real (kind=real4), dimension(:,:), allocatable, public,target, save:: re_vis
     real (kind=real4), dimension(:,:), allocatable, public,target, save:: iwp_vis
     real (kind=real4), dimension(:,:), allocatable, public,target, save:: iwp_tau_vis
     real (kind=real4), dimension(:,:), allocatable, public,target, save:: lwp_vis
     real (kind=real4), dimension(:,:), allocatable, public,target, save:: H_vis
     real (kind=real4), dimension(:,:), allocatable, public,target, save:: N_vis
     integer (kind=int1), dimension(:,:), allocatable, public,target, save:: tau_vis_qf
     integer (kind=int1), dimension(:,:), allocatable, public,target, save:: re_vis_qf
     integer (kind=int1), dimension(:,:), allocatable, public,target, save:: small_particle_tau_re_qf
     integer (kind=int1), dimension(:,:), allocatable, public,target, save:: large_particle_tau_re_qf

!--- pixel level aerosol props
     real (kind=real4), dimension(:,:), allocatable, public, save:: aot1
     real (kind=real4), dimension(:,:), allocatable, public, save:: aot2
     real (kind=real4), dimension(:,:), allocatable, public, save:: aot3a
     integer (kind=int1), dimension(:,:), allocatable, public, save:: aot_qf


!--- non-cloud properties
     integer (kind=int1), dimension(:,:), allocatable, public, save:: sst_qf
     integer (kind=int1), dimension(:,:), allocatable, public, save:: ndvi_qf
     integer (kind=int1), dimension(:,:), allocatable, public, save:: tsfc_qf

!--- pixel level radiative flux props
     real (kind=real4), dimension(:,:), allocatable, public, save:: olr
     real (kind=real4), dimension(:,:), allocatable, public, save:: swar
     integer (kind=int1), dimension(:,:), allocatable, public, save:: olr_qf
     integer (kind=int1), dimension(:,:), allocatable, public, save:: swar_qf
     real (kind=real4), dimension(:,:), allocatable, public, save:: cloud_063um_albedo
     real (kind=real4), dimension(:,:), allocatable, public, save:: cloud_063um_spherical_albedo
     real (kind=real4), dimension(:,:), allocatable, public, save:: cloud_063um_transmission_view
     real (kind=real4), dimension(:,:), allocatable, public, save:: cloud_063um_transmission_solar

     !---- dcomp params	 
     real (kind=real4), dimension(:,:), allocatable, public, save:: cost_dcomp
     real (kind=real4), dimension(:,:), allocatable, public, save:: error_cov_matrix_cod
     real (kind=real4), dimension(:,:), allocatable, public, save:: error_cov_matrix_ref
     real (kind=real4), dimension(:,:), allocatable, public, save:: dcomp_atmos_correct_AC_VIS
     real (kind=real4), dimension(:,:), allocatable, public, save:: dcomp_atmos_correct_AC_IR
     
   

     !--- pixel level radiative flux props
     real (kind=real4), dimension(:,:), allocatable, public, save:: rsr
     integer (kind=int1), dimension(:,:), allocatable, public, save:: rsr_qf

     !-- pixel level atmospheric reflectance components
  real (kind=real4), dimension(:,:), allocatable, public:: alb1_clear
  real (kind=real4), dimension(:,:), allocatable, public:: alb2_clear
  real (kind=real4), dimension(:,:), allocatable, public:: alb3a_clear
  real (kind=real4), dimension(:,:), allocatable, public:: alb1_sfc
  real (kind=real4), dimension(:,:), allocatable, public:: alb2_sfc
  real (kind=real4), dimension(:,:), allocatable, public:: alb3a_sfc
  real (kind=real4), dimension(:,:), allocatable, public:: alb3b_sfc
  real (kind=real4), dimension(:,:), allocatable, public:: rad3b_sfc
  real (kind=real4), dimension(:,:), allocatable, public:: rad4_sfc
  real (kind=real4), dimension(:,:), allocatable, public:: rad5_sfc
  real (kind=real4), dimension(:,:), allocatable, public:: ndvi_sfc
  real (kind=real4), dimension(:,:), allocatable, public:: bt4_sfc
  real (kind=real4), dimension(:,:), allocatable, public:: rad3b_ems_sfc
  real (kind=real4), dimension(:,:), allocatable, public:: ems3b_sfc

!--- integers values for output to files
  integer(kind=int1), dimension(:,:), public,save,allocatable:: one_byte_temp
  integer(kind=int2), dimension(:,:), public,save,allocatable:: two_byte_temp
  integer(kind=int1), dimension(:,:), public,save,allocatable:: one_byte_anchor_temp
  integer(kind=int2), dimension(:,:), public,save,allocatable:: two_byte_anchor_temp

!---volcanic ash parameters
  real(kind=real4), allocatable, dimension(:,:), public, save:: ash_probability
  real(kind=real4), allocatable, dimension(:,:), public, save:: ash_mass_loading
  real(kind=real4), allocatable, dimension(:,:), public, save:: ash_height
  real(kind=real4), allocatable, dimension(:,:), public, save:: ash_pressure
  real(kind=real4), allocatable, dimension(:,:), public, save:: ash_temperature
  real(kind=real4), allocatable, dimension(:,:), public, save:: ash_ir_optical_depth
  real(kind=real4), allocatable, dimension(:,:), public, save:: ash_vis_optical_depth
  real(kind=real4), allocatable, dimension(:,:), public, save:: ash_effective_radius
  real(kind=real4), allocatable, dimension(:,:), public, save:: ash_emiss
  integer(kind=int1), allocatable, dimension(:,:), public, save:: ash_temperature_qf
  integer(kind=int1), allocatable, dimension(:,:), public, save:: ash_emiss_qf
  integer(kind=int1), allocatable, dimension(:,:), public, save:: ash_beta1112_qf
  real(kind=real4), allocatable, dimension(:,:), public, save:: ash_emiss_tot
  real(kind=real4), allocatable, dimension(:,:), public, save:: ash_beta1112
  real(kind=real4), allocatable, dimension(:,:), public, save:: ash_beta1112_tot
  real(kind=real4), allocatable, dimension(:,:), public, save:: ash_beta1112_opaque
  real(kind=real4), allocatable, dimension(:,:), public, save:: ash_beta1112_tot_multi_low
  real(kind=real4), allocatable, dimension(:,:), public, save:: ash_beta1112_tot_multi_mid
  real(kind=real4), allocatable, dimension(:,:), public, save:: ash_generic1
  real(kind=real4), allocatable, dimension(:,:), public, save:: ash_generic2
  real(kind=real4), allocatable, dimension(:,:), public, save:: ash_emissivity_error
  real(kind=real4), allocatable, dimension(:,:), public, save:: ash_temperature_error
  real(kind=real4), allocatable, dimension(:,:), public, save:: ash_beta1112_error
  integer(kind=int1), allocatable, dimension(:,:), public, save:: ash_qf
  integer(kind=int2), allocatable, dimension(:,:), public, save:: ash_mask
  integer(kind=int4), allocatable, dimension(:,:), public, save:: ash_x_lrc
  integer(kind=int4), allocatable, dimension(:,:), public, save:: ash_y_lrc

!--- nwp parameters
 integer, allocatable, dimension(:,:), public, save :: i_nwp
 integer, allocatable, dimension(:,:), public, save :: j_nwp
 integer, allocatable, dimension(:,:), public, save :: ivza_rtm
 integer, allocatable, dimension(:,:), public, save :: i_nwp_x
 integer, allocatable, dimension(:,:), public, save :: j_nwp_x
 real, allocatable, dimension(:,:), public, save :: lon_nwp_fac
 real, allocatable, dimension(:,:), public, save :: lat_nwp_fac

!--- local radiative center
integer, allocatable, dimension(:,:), public, save :: i_lrc
integer, allocatable, dimension(:,:), public, save :: j_lrc

!--- rtm parameters
  real (kind=real4), dimension(:,:), allocatable, public:: rad_clear_ch3b_rtm
  real (kind=real4), dimension(:,:), allocatable, public:: rad_clear_ch4_rtm
  real (kind=real4), dimension(:,:), allocatable, public:: rad_clear_ch5_rtm
  real (kind=real4), dimension(:,:), allocatable, public:: bt_clear_ch3b_rtm
  real (kind=real4), dimension(:,:), allocatable, public:: bt_clear_ch4_rtm
  real (kind=real4), dimension(:,:), allocatable, public:: bt_clear_ch5_rtm
  real (kind=real4), dimension(:,:), allocatable, public:: rad_atm_ch3b_rtm
  real (kind=real4), dimension(:,:), allocatable, public:: rad_atm_ch4_rtm
  real (kind=real4), dimension(:,:), allocatable, public:: rad_atm_ch5_rtm
  real (kind=real4), dimension(:,:), allocatable, public:: trans_atm_ch3b_rtm
  real (kind=real4), dimension(:,:), allocatable, public:: trans_atm_ch4_rtm
  real (kind=real4), dimension(:,:), allocatable, public:: trans_atm_ch5_rtm
  real (kind=real4), dimension(:,:), allocatable, public:: tsfc_rtm
  real (kind=real4), dimension(:,:), allocatable, public:: tpw_rtm
  real (kind=real4), dimension(:,:), allocatable, public:: trans_atm_ch3b_solar_total_rtm
  real (kind=real4), dimension(:,:), allocatable, public:: trans_atm_ch3b_solar_rtm
  real (kind=real4), dimension(:,:), allocatable, public:: bt_clear_ch3b_solar_rtm
  real (kind=real4), dimension(:,:), allocatable, public:: rad_clear_ch3b_solar_rtm
  real (kind=real4), dimension(:,:), allocatable, public:: ems3b_clear_rtm
  real (kind=real4), dimension(:,:), allocatable, public:: ems3b_clear_solar_rtm
  real (kind=real4), dimension(:,:), allocatable, public:: ems3b_clear_solar_sfc_rtm
  real (kind=real4), dimension(:,:), allocatable, public:: ttropo_rtm
  real (kind=real4), dimension(:,:), allocatable, public:: emiss_11_tropo_rtm
  real (kind=real4), dimension(:,:), allocatable, public:: emiss_11_tropo_nadir_rtm
  real (kind=real4), dimension(:,:), allocatable, public:: beta_11_12_tropo_rtm
  real (kind=real4), dimension(:,:), allocatable, public:: bt_clear_ch4_rtm_unbiased
  real (kind=real4), dimension(:,:), allocatable, public:: bt_clear_ch5_rtm_unbiased
  real (kind=real4), dimension(:,:), allocatable, public:: rad_clear_ch4_rtm_unbiased
  real (kind=real4), dimension(:,:), allocatable, public:: rad_clear_ch5_rtm_unbiased


  !--- ir cld layer variables
  real (kind=real4), dimension(:,:), allocatable, public:: beta_11_12_850mb_rtm
  real (kind=real4), dimension(:,:), allocatable, public:: beta_11_12_mean_ec
  real (kind=real4), dimension(:,:), allocatable, public:: Pc_opaque_cloud
  real (kind=real4), dimension(:,:), allocatable, public:: Zc_opaque_cloud
  real (kind=real4), dimension(:,:), allocatable, public:: Tc_opaque_cloud

  real (kind=real4), dimension(:,:), allocatable, public:: Pc_lower_cloud
  real (kind=real4), dimension(:,:), allocatable, public:: Zc_lower_cloud
  real (kind=real4), dimension(:,:), allocatable, public:: Tc_lower_cloud

  !--- cloud layer variables
  integer (kind=int1), dimension(:,:), allocatable, public:: max_cloud_rtm_level

!--- modis white sky albedo maps
  real (kind=real4), dimension(:,:), allocatable, public:: alb1_sfc_white_sky
  real (kind=real4), dimension(:,:), allocatable, public:: alb3a_sfc_white_sky

!---- other static arrays carried by this module
  real, dimension(20,3,3), public, save:: h2o_tau_coef
  real, dimension(20,3), public, save:: tau_ray_corr
  real, dimension(20,3), public, save:: tau_h2o_corr
  real, dimension(20,3), public, save:: tau_o3_corr
  real, dimension(20,3), public, save:: tau_o2_corr
  real, dimension(20,3), public, save:: tau_ch4_corr
  real, dimension(20,3), public, save:: tau_co2_corr

!--- pixel arrays used in clavr_mod
  real(kind=real4), dimension(:,:), allocatable, public:: pix_data

!--- flags for using clavrxorb_default_file
  integer ,public, save :: use_default

!--- clavrxorb_file_list filename
  character(len=300), public, save:: file_list

 contains

!----------------------------------------------------------------------------
! This routine allocate the memory for the pixel arrays 
!----------------------------------------------------------------------------
subroutine CREATE_PIXEL_ARRAYS(nchan_clavr)


   integer,intent(in):: nchan_clavr

   allocate(buffer(l1b_rec_length))
   allocate(segment_buffer(l1b_rec_length*(j_max-j_min+1)))
   allocate(header_buffer(l1b_rec_length))

   allocate(alb1(num_pix,j_min:j_max),alb2(num_pix,j_min:j_max), &
          alb3a(num_pix,j_min:j_max),alb3b(num_pix,j_min:j_max), &
          rad3b(num_pix,j_min:j_max), &
          rad4(num_pix,j_min:j_max),rad5(num_pix,j_min:j_max), &
          rad3b_ems(num_pix,j_min:j_max),bt3b(num_pix,j_min:j_max), &
          bt4(num_pix,j_min:j_max),bt5(num_pix,j_min:j_max),&
          solzen(num_pix,j_min:j_max), &
          lat(num_pix,j_min:j_max), &
          lon(num_pix,j_min:j_max), &
          lat_1b(num_pix,j_min:j_max), &
          lon_1b(num_pix,j_min:j_max), &
          glintzen(num_pix,j_min:j_max), &
          airmass(num_pix,j_min:j_max),scatangle(num_pix,j_min:j_max), &
          land(num_pix,j_min:j_max),coast(num_pix,j_min:j_max), &
          land_mask(num_pix,j_min:j_max), &
          coast_mask(num_pix,j_min:j_max), &
          glint_mask(num_pix,j_min:j_max), &
          ems3b(num_pix,j_min:j_max),  &
          bad_pixel_mask(num_pix,j_min:j_max), &
          ch3a_on_pixel_mask(num_pix,j_min:j_max), &
          volcano_mask(num_pix,j_min:j_max),&
          coast_mask_nwp(num_pix,j_min:j_max), &
          space_mask(num_pix,j_min:j_max))

   allocate(solar_contamination_mask(num_pix,j_min:j_max))
   allocate(satzen(num_pix,j_min:j_max))
   allocate(relaz(num_pix,j_min:j_max))
   allocate(solaz(num_pix,j_min:j_max))
   allocate(rut(num_pix,j_min:j_max))
   allocate(tut(num_pix,j_min:j_max))


!--- 3x3 uni
   allocate &
          (bt4_mean_3x3(num_pix,j_min:j_max),bt4_max_3x3(num_pix,j_min:j_max),     &
           bt4_min_3x3(num_pix,j_min:j_max),bt4_std_3x3(num_pix,j_min:j_max),      &
           alb1_mean_3x3(num_pix,j_min:j_max),alb1_max_3x3(num_pix,j_min:j_max),   &
           alb1_min_3x3(num_pix,j_min:j_max),alb1_std_3x3(num_pix,j_min:j_max),    &
           bt3b_mean_3x3(num_pix,j_min:j_max),bt3b_max_3x3(num_pix,j_min:j_max),   &
           bt3b_min_3x3(num_pix,j_min:j_max),bt3b_std_3x3(num_pix,j_min:j_max),    &
           zsfc_mean_3x3(num_pix,j_min:j_max),zsfc_max_3x3(num_pix,j_min:j_max),   &
           zsfc_min_3x3(num_pix,j_min:j_max),zsfc_std_3x3(num_pix,j_min:j_max),    &
           ems3b_mean_3x3(num_pix,j_min:j_max),ems3b_max_3x3(num_pix,j_min:j_max), &
           ems3b_min_3x3(num_pix,j_min:j_max),ems3b_std_3x3(num_pix,j_min:j_max),  &
           i_max_bt4_3x3(num_pix,j_min:j_max),j_max_bt4_3x3(num_pix,j_min:j_max),  &
           i_min_bt4_3x3(num_pix,j_min:j_max),j_min_bt4_3x3(num_pix,j_min:j_max))

   allocate(bt45_mean_3x3(num_pix,j_min:j_max), &
            bt45_max_3x3(num_pix,j_min:j_max), &
            bt45_min_3x3(num_pix,j_min:j_max), &
            bt45_std_3x3(num_pix,j_min:j_max))
 
   allocate(bt45_bt4_max_3x3(num_pix,j_min:j_max))

   allocate(alb1_sfc_white_sky_mean_3x3(num_pix,j_min:j_max))
   allocate(alb1_sfc_white_sky_max_3x3(num_pix,j_min:j_max))
   allocate(alb1_sfc_white_sky_min_3x3(num_pix,j_min:j_max))
   allocate(alb1_sfc_white_sky_std_3x3(num_pix,j_min:j_max))

   allocate(ems3b_median_3x3(num_pix,j_min:j_max))
   allocate(ems3b_std_median_3x3(num_pix,j_min:j_max))
   allocate(bt4_median_3x3(num_pix,j_min:j_max))
   allocate(bt4_std_median_3x3(num_pix,j_min:j_max))
   allocate(bt3b_median_3x3(num_pix,j_min:j_max))
   allocate(bt3b_std_median_3x3(num_pix,j_min:j_max))

   allocate(cloud_fraction_3x3(num_pix,j_min:j_max))
   allocate(cloud_fraction_uncer_3x3(num_pix,j_min:j_max))

 allocate(   &
          desert_mask(num_pix,j_min:j_max), &
          snow_hires(num_pix,j_min:j_max), &
          snow(num_pix,j_min:j_max), &
          dust(num_pix,j_min:j_max), &
          smoke(num_pix,j_min:j_max), &
          fire(num_pix,j_min:j_max), &
          sfc_type(num_pix,j_min:j_max), &
          zsfc(num_pix,j_min:j_max), &
          seczen(num_pix,j_min:j_max), &
          coszen(num_pix,j_min:j_max), &
          cossolzen(num_pix,j_min:j_max), &
          sst_anal(num_pix,j_min:j_max), &
          sst_anal_err(num_pix,j_min:j_max), &
          sst_anal_cice(num_pix,j_min:j_max), &
          sst_anal_uni(num_pix,j_min:j_max), &
          sst_qf(num_pix,j_min:j_max), &
          ndvi_qf(num_pix,j_min:j_max), &
          tsfc(num_pix,j_min:j_max), &
          tsfc_qf(num_pix,j_min:j_max), &
          temp_pix_array(num_pix,j_min:j_max), &
          temp_pix_array_2(num_pix,j_min:j_max), &
          pix_array_1(num_pix,j_min:j_max), &
          pix_array_2(num_pix,j_min:j_max))

  allocate(emiss_11_tropo_rtm(num_pix,j_min:j_max))
  allocate(emiss_11_tropo_nadir_rtm(num_pix,j_min:j_max))
  allocate(beta_11_12_tropo_rtm(num_pix,j_min:j_max))

  !---  ir cloud layer
  allocate(beta_11_12_850mb_rtm(num_pix,j_min:j_max))
  allocate(beta_11_12_mean_ec(num_pix,j_min:j_max))
  allocate(Pc_opaque_cloud(num_pix,j_min:j_max))
  allocate(Zc_opaque_cloud(num_pix,j_min:j_max))
  allocate(Tc_opaque_cloud(num_pix,j_min:j_max))

  if (cld_flag == sym%YES) then

  !split window
  allocate(Tc_sw(num_pix,j_min:j_max))
  allocate(ec_sw(num_pix,j_min:j_max))
  allocate(tau_sw(num_pix,j_min:j_max))
  allocate(Tc_sw_error(num_pix,j_min:j_max))
  allocate(ec_sw_error(num_pix,j_min:j_max))
  allocate(beta_sw_error(num_pix,j_min:j_max))
  allocate(Tc_sw_qf(num_pix,j_min:j_max))
  allocate(ec_sw_qf(num_pix,j_min:j_max))
  allocate(beta_sw(num_pix,j_min:j_max))
  allocate(Pc_sw(num_pix,j_min:j_max))
  allocate(Zc_sw(num_pix,j_min:j_max))
  allocate(cld_layer_sw(num_pix,j_min:j_max))
  allocate(beta_sw_qf(num_pix,j_min:j_max))

!visible
  allocate(tau_vis(num_pix,j_min:j_max))
  allocate(tau_vis_ap(num_pix,j_min:j_max))
  allocate(vis_ref_fm(num_pix,j_min:j_max))
  allocate(re_vis(num_pix,j_min:j_max))
  allocate(lwp_vis(num_pix,j_min:j_max))
  allocate(iwp_vis(num_pix,j_min:j_max))
  allocate(iwp_tau_vis(num_pix,j_min:j_max))
  allocate(H_vis(num_pix,j_min:j_max))
  allocate(N_vis(num_pix,j_min:j_max))
  allocate(tau_vis_qf(num_pix,j_min:j_max))
  allocate(re_vis_qf(num_pix,j_min:j_max))
  allocate(small_particle_tau_re_qf(num_pix,j_min:j_max))
  allocate(large_particle_tau_re_qf(num_pix,j_min:j_max))
  allocate(cloud_063um_albedo(num_pix,j_min:j_max))
  allocate(cloud_063um_spherical_albedo(num_pix,j_min:j_max))
  allocate(cloud_063um_transmission_view(num_pix,j_min:j_max))
  allocate(cloud_063um_transmission_solar(num_pix,j_min:j_max))
  allocate(cost_dcomp(num_pix,j_min:j_max))
  allocate(error_cov_matrix_cod(num_pix,j_min:j_max))
  allocate(error_cov_matrix_ref(num_pix,j_min:j_max))
  allocate(dcomp_atmos_correct_AC_VIS(num_pix,j_min:j_max))
  allocate(dcomp_atmos_correct_AC_IR(num_pix,j_min:j_max))
  
  allocate(Pc_lower_cloud(num_pix,j_min:j_max))
  allocate(Zc_lower_cloud(num_pix,j_min:j_max))
  allocate(Tc_lower_cloud(num_pix,j_min:j_max))

  endif

!---ash
  if (ash_flag == sym%YES) then
    allocate(ash_probability(num_pix,j_min:j_max))
    allocate(ash_mass_loading(num_pix,j_min:j_max))
    allocate(ash_height(num_pix,j_min:j_max))
    allocate(ash_pressure(num_pix,j_min:j_max))
    allocate(ash_temperature(num_pix,j_min:j_max))
    allocate(ash_ir_optical_depth(num_pix,j_min:j_max))
    allocate(ash_vis_optical_depth(num_pix,j_min:j_max))
    allocate(ash_effective_radius(num_pix,j_min:j_max))
    allocate(ash_emiss(num_pix,j_min:j_max))
    allocate(ash_temperature_qf(num_pix,j_min:j_max))
    allocate(ash_emiss_qf(num_pix,j_min:j_max))
    allocate(ash_beta1112_qf(num_pix,j_min:j_max))
    allocate(ash_emiss_tot(num_pix,j_min:j_max))
    allocate(ash_beta1112(num_pix,j_min:j_max))
    allocate(ash_beta1112_tot(num_pix,j_min:j_max))
    allocate(ash_beta1112_opaque(num_pix,j_min:j_max))
    allocate(ash_beta1112_tot_multi_low(num_pix,j_min:j_max))
    allocate(ash_beta1112_tot_multi_mid(num_pix,j_min:j_max))
    allocate(ash_generic1(num_pix,j_min:j_max))
    allocate(ash_generic2(num_pix,j_min:j_max))
    allocate(ash_emissivity_error(num_pix,j_min:j_max))
    allocate(ash_temperature_error(num_pix,j_min:j_max))
    allocate(ash_beta1112_error(num_pix,j_min:j_max))
    allocate(ash_qf(num_pix,j_min:j_max))
    allocate(ash_mask(num_pix,j_min:j_max))
    allocate(ash_x_lrc(num_pix,j_min:j_max))
    allocate(ash_y_lrc(num_pix,j_min:j_max))
  endif

!--- aerosol
  if (aer_flag == sym%YES) then
     allocate(aot1(num_pix,j_min:j_max))
     allocate(aot2(num_pix,j_min:j_max))
     allocate(aot3a(num_pix,j_min:j_max))
     allocate(aot_qf(num_pix,j_min:j_max))
  endif

!--- earth radiation budget
  if (erb_flag == sym%YES) then
      allocate(olr(num_pix,j_min:j_max))
      allocate(olr_qf(num_pix,j_min:j_max))
      allocate(swar(num_pix,j_min:j_max))
      allocate(swar_qf(num_pix,j_min:j_max))
  endif

  allocate(chan_counts(5,num_pix,j_min:j_max))
  allocate(scan_space_counts(5,j_min:j_max))
  allocate(scan_bb_counts(5,j_min:j_max))
  allocate(chan_counts_sg(5,num_pix,j_min:j_max))

  !--- thermal calibration
  allocate(ir_coef_1_1b(3:5,j_min:j_max))
  allocate(ir_coef_2_1b(3:5,j_min:j_max))
  allocate(ir_coef_3_1b(3:5,j_min:j_max))
  allocate(ir_linear_slope_1b(3:5,j_min:j_max))
  allocate(ir_linear_intercept_1b(3:5,j_min:j_max))
  allocate(ir_linear_slope_new(3:5,j_min:j_max))
  allocate(ir_linear_intercept_new(3:5,j_min:j_max))

  !--- reflectance calibration
  allocate(vis_slope_1(1:3,j_min:j_max))
  allocate(vis_slope_2(1:3,j_min:j_max))
  allocate(vis_intercept_1(1:3,j_min:j_max))
  allocate(vis_intercept_2(1:3,j_min:j_max))
  allocate(vis_intersection(1:3,j_min:j_max))

  allocate(cld_mask_1b(num_pix,j_min:j_max))
  allocate(cld_mask(num_pix,j_min:j_max))
  allocate(cld_mask_ir(num_pix,j_min:j_max))
  allocate(cld_mask_qf(num_pix,j_min:j_max))
  allocate(cld_mask_bayes(num_pix,j_min:j_max))
  allocate(cld_phase_bayes(num_pix,j_min:j_max))
  allocate(posterior_cld_probability(num_pix,j_min:j_max))
  allocate(posterior_phase_probability(num_pix,j_min:j_max))
  allocate(cld_type(num_pix,j_min:j_max))
  allocate(cld_type_ir(num_pix,j_min:j_max))
  allocate(cld_type_bayes(num_pix,j_min:j_max))
  allocate(cld_type_inoue(num_pix,j_min:j_max))
  allocate(max_cloud_rtm_level(num_pix,j_min:j_max))
  allocate(temp_mask(num_pix,j_min:j_max))
  allocate(cirrus_quality(num_pix,j_min:j_max))
  allocate(cld_test_vector(max_num_cld_tests,num_pix,j_min:j_max))
  allocate(cld_test_vector_packed(max_num_cld_test_bytes,num_pix,j_min:j_max))


  allocate(sst_mask(num_pix,j_min:j_max))
  allocate(sst_oper(num_pix,j_min:j_max))
  allocate(sst_masked(num_pix,j_min:j_max))
  allocate(ndvi(num_pix,j_min:j_max))
  allocate(bt45(num_pix,j_min:j_max))
  allocate(bt3b4(num_pix,j_min:j_max))
  allocate(bt3b5(num_pix,j_min:j_max))
  allocate(alb21(num_pix,j_min:j_max))
  allocate(alb31(num_pix,j_min:j_max))
  allocate(ndsi(num_pix,j_min:j_max))
 
  allocate(dcc_mask(num_pix,j_min:j_max))

  !--- anchor points
  allocate(lon_anchor_1b(num_anchors,j_min:j_max))
  allocate(lat_anchor_1b(num_anchors,j_min:j_max))
  allocate(solzen_anchor(num_anchors,j_min:j_max))
  allocate(satzen_anchor(num_anchors,j_min:j_max))
  allocate(relaz_anchor(num_anchors,j_min:j_max))
  allocate(solaz_anchor(num_anchors,j_min:j_max))
  allocate(glintzen_anchor(num_anchors,j_min:j_max))
  allocate(scatangle_anchor(num_anchors,j_min:j_max))
  
  allocate(one_byte_temp(num_pix,j_min:j_max))
  allocate(two_byte_temp(num_pix,j_min:j_max))
  allocate(one_byte_anchor_temp(num_anchors,j_min:j_max))
  allocate(two_byte_anchor_temp(num_anchors,j_min:j_max))

  !--- atmospherically corrected parameters
  allocate(alb1_clear(num_pix,j_min:j_max))
  allocate(alb2_clear(num_pix,j_min:j_max))
  allocate(alb3a_clear(num_pix,j_min:j_max))
  allocate(alb1_sfc(num_pix,j_min:j_max))
  allocate(alb2_sfc(num_pix,j_min:j_max))
  allocate(alb3a_sfc(num_pix,j_min:j_max))
  allocate(ndvi_sfc(num_pix,j_min:j_max))
  allocate(alb3b_sfc(num_pix,j_min:j_max))
  allocate(rad3b_sfc(num_pix,j_min:j_max))
  allocate(rad4_sfc(num_pix,j_min:j_max))
  allocate(rad5_sfc(num_pix,j_min:j_max))
  allocate(bt4_sfc(num_pix,j_min:j_max))
  allocate(rad3b_ems_sfc(num_pix,j_min:j_max))
  allocate(ems3b_sfc(num_pix,j_min:j_max))
  allocate(rsr(num_pix,j_min:j_max))
  allocate(rsr_qf(num_pix,j_min:j_max))

  !--- pixel level parameters
   allocate(alb1_sfc_white_sky(num_pix,j_min:j_max))
   allocate(alb3a_sfc_white_sky(num_pix,j_min:j_max))

   allocate(i_nwp(num_pix,j_min:j_max))
   allocate(j_nwp(num_pix,j_min:j_max))
   allocate(ivza_rtm(num_pix,j_min:j_max))
   allocate(i_nwp_x(num_pix,j_min:j_max))
   allocate(j_nwp_x(num_pix,j_min:j_max))
   allocate(lat_nwp_fac(num_pix,j_min:j_max))
   allocate(lon_nwp_fac(num_pix,j_min:j_max))


   allocate(i_lrc(num_pix,j_min:j_max))
   allocate(j_lrc(num_pix,j_min:j_max))

  allocate(fatal(j_min:j_max),ascend(j_min:j_max), &
           bad_scan_flag(j_min:j_max),ch3a_on(j_min:j_max), &
           resync(j_min:j_max),clavr_status_1b(j_min:j_max),&
           clavr_status(j_min:j_max),scan_number(j_min:j_max), &
           scan_time(j_min:j_max), &
           scan_day(j_min:j_max), &
           scan_year(j_min:j_max), &
           utc_scan_time_hours(j_min:j_max), &
           pixel_local_time_hours(num_pix,j_min:j_max), &
           pixel_time(num_pix,j_min:j_max))

  allocate(pix_data(nchan_clavr,num_pix))

  allocate(rad_clear_ch3b_solar_rtm(num_pix,j_min:j_max), &
           rad_clear_ch3b_rtm(num_pix,j_min:j_max), &
           rad_clear_ch4_rtm(num_pix,j_min:j_max), &
           rad_clear_ch5_rtm(num_pix,j_min:j_max), &
           bt_clear_ch3b_solar_rtm(num_pix,j_min:j_max), &
           bt_clear_ch3b_rtm(num_pix,j_min:j_max), &
           bt_clear_ch4_rtm(num_pix,j_min:j_max), &
           bt_clear_ch5_rtm(num_pix,j_min:j_max), &
           rad_atm_ch3b_rtm(num_pix,j_min:j_max), &
           trans_atm_ch3b_rtm(num_pix,j_min:j_max), &
           trans_atm_ch3b_solar_total_rtm(num_pix,j_min:j_max), &
           trans_atm_ch3b_solar_rtm(num_pix,j_min:j_max), &
           rad_atm_ch4_rtm(num_pix,j_min:j_max), &
           trans_atm_ch4_rtm(num_pix,j_min:j_max), &
           rad_atm_ch5_rtm(num_pix,j_min:j_max), &
           trans_atm_ch5_rtm(num_pix,j_min:j_max), &
           tsfc_rtm(num_pix,j_min:j_max), &
           tpw_rtm(num_pix,j_min:j_max), &
           ttropo_rtm(num_pix,j_min:j_max), &
           ems3b_clear_rtm(num_pix,j_min:j_max), &
           ems3b_clear_solar_rtm(num_pix,j_min:j_max), &
           ems3b_clear_solar_sfc_rtm(num_pix,j_min:j_max), &
           bt_clear_ch4_rtm_unbiased(num_pix,j_min:j_max), &
           bt_clear_ch5_rtm_unbiased(num_pix,j_min:j_max), &
           rad_clear_ch4_rtm_unbiased(num_pix,j_min:j_max), &
           rad_clear_ch5_rtm_unbiased(num_pix,j_min:j_max))


!-------------------------------------------
   scan_number = 0
   scan_time = 0
   scan_day = 0
   scan_year = 0
   utc_scan_time_hours = 0.0
   pixel_local_time_hours = 0.0
   pixel_time = 0.0
   fatal = 0
   ascend = 0
   bad_scan_flag = 0
   ch3a_on = 0
   resync = 0

   clavr_status_1b = 0
   clavr_status = 0
   alb1 = 0.0
   alb2 = 0.0
   alb3a = 0.0
   alb3b = 0.0
   rad3b = 0.0
   rad3b_ems = 0.0
   ems3b = 0.0
   bt3b = 0.0
   rad4 = 0.0
   bt4 = 0.0
   rad5 = 0.0
   bt5 = 0.0
   lat_1b = 0.0
   lon_1b = 0.0
   lat = 0.0
   lon = 0.0

   cld_mask_1b = 0
   cld_mask = 0
   cld_mask_ir = 0
   cld_mask_qf = 0
   cld_mask_bayes = 0
   cld_phase_bayes = 0
   posterior_cld_probability = 0
   posterior_phase_probability = 0
   temp_mask = 0
   clavr_status_1b = 0
   clavr_status = 0
   cld_type = 0
   cld_type_ir = 0
   cld_type_bayes = 0
   cld_type_inoue = 0
   max_cloud_rtm_level = 0
   cirrus_quality = 0
   desert_mask = 0
   sfc_type = 0
   bad_pixel_mask = sym%NO
   ch3a_on_pixel_mask = sym%NO
   solar_contamination_mask = sym%NO
   volcano_mask = sym%NO
   land = sym%LAND
   coast = sym%NO_COAST
   space_mask = sym%SPACE
   land_mask = sym%NO
   glint_mask = sym%NO
   coast_mask = sym%NO
   coast_mask_nwp = sym%NO
   zsfc = 0.0
   lat_anchor_1b = 0.0
   lon_anchor_1b = 0.0
   solzen_anchor = 0.0
   satzen_anchor = 0.0
   relaz_anchor = 0.0
   solaz_anchor = 0.0
   glintzen_anchor = 0.0
   scatangle_anchor = 0.0

   rut = 0
   tut = 0

   solzen = 0.0
   satzen = 0.0
   relaz = 0.0
   solaz = 0.0
   scatangle = 0.0
   airmass = 0.0
   cld_test_vector = 0
   cld_test_vector_packed = 0
   dcc_mask = sym%NO
   sst_mask = 0.0
   sst_oper = 0.0
   sst_masked = 0.0
   ndvi = 0.0
   ndsi = 0.0
   snow = sym%NO_SNOW
   snow_hires = sym%NO_SNOW
   dust = 0
   smoke = 0
   fire = 0

   bt45 = 0.0
   bt3b4 = 0.0
   bt3b5 = 0.0
   alb21 = 0.0
   alb31 = 0.0  
   sst_anal = 0.0
   sst_anal_err = 0.0
   sst_anal_cice = 0.0
   sst_anal_uni = 0.0
   tsfc = 0.0
   sst_qf = 0
   tsfc_qf = 0
   ndvi_qf = 0
   temp_pix_array = 0.0
   temp_pix_array_2 = 0.0
   pix_array_1 = 0.0
   pix_array_2 = 0.0

   alb1_clear = 0.0
   alb2_clear = 0.0
   alb3a_clear = 0.0
   alb1_sfc = 0.0
   alb2_sfc = 0.0
   alb3a_sfc = 0.0
   ndvi_sfc = 0.0
   alb3b_sfc = 0.0
   rad3b_sfc = 0.0
   rad4_sfc = 0.0
   rad5_sfc = 0.0
   bt4_sfc = 0.0
   rad3b_sfc = 0.0
   ems3b_sfc = 0.0
   alb1_sfc_white_sky = 0.0
   alb3a_sfc_white_sky = 0.0
   rsr = 0.0
   rsr_qf = 0.0

   if (aer_flag == sym%YES) then
    aot1 = 0.0
    aot_qf = 0
    aot2 = 0.0
    aot3a = 0.0
   endif

!--- ash
  if (ash_flag == sym%YES) then
    ash_probability = 0.0
    ash_mass_loading = 0.0
    ash_height = 0.0
    ash_pressure = 0.0
    ash_temperature = 0.0
    ash_ir_optical_depth = 0.0
    ash_vis_optical_depth = 0.0
    ash_effective_radius = 0.0
    ash_emiss = 0.0
    ash_temperature_qf = 0
    ash_emiss_qf = 0
    ash_beta1112_qf = 0
    ash_emiss_tot = 0.0
    ash_beta1112 = 0.0
    ash_beta1112_tot = 0.0
    ash_beta1112_opaque = 0.0
    ash_beta1112_tot_multi_low = 0.0
    ash_beta1112_tot_multi_mid = 0.0
    ash_generic1 = 0.0
    ash_generic2 = 0.0
    ash_emissivity_error = 0.0
    ash_temperature_error = 0.0
    ash_beta1112_error = 0.0
    ash_qf = 0
    ash_mask = 0
    ash_x_lrc = 0
    ash_y_lrc = 0
  endif


!--- earth radiation budget
  if (erb_flag == sym%YES) then
   olr = 0.0
   olr_qf = 0
   swar = 0.0
   swar_qf = 0
  endif

  chan_counts = 0
  chan_counts_sg = 0
  scan_space_counts=0
  scan_bb_counts=0
  buffer = 0
  segment_buffer = 0
  header_buffer = 0

  one_byte_temp = 0
  two_byte_temp = 0
  one_byte_anchor_temp = 0
  two_byte_anchor_temp = 0
  temp_pix_array = 0.0

  i_lrc = 0
  j_lrc = 0

  i_nwp = 0
  j_nwp = 0
  ivza_rtm = 0
  i_nwp_x = 0
  j_nwp_x = 0
  lon_nwp_fac = 0.0
  lat_nwp_fac = 0.0

  pix_data = 0.0

  rad_clear_ch3b_solar_rtm = 0.0
  rad_clear_ch3b_rtm = 0.0
  rad_clear_ch4_rtm = 0.0
  rad_clear_ch5_rtm = 0.0
  bt_clear_ch3b_solar_rtm = 0.0
  bt_clear_ch3b_rtm = 0.0
  bt_clear_ch4_rtm = 0.0
  bt_clear_ch5_rtm = 0.0
  rad_atm_ch3b_rtm = 0.0
  rad_atm_ch4_rtm = 0.0
  rad_atm_ch5_rtm = 0.0
  trans_atm_ch3b_rtm = 0.0
  trans_atm_ch3b_solar_rtm = 0.0
  trans_atm_ch3b_solar_total_rtm = 0.0
  trans_atm_ch4_rtm = 0.0
  trans_atm_ch5_rtm = 0.0
  tsfc_rtm = 0.0
  tpw_rtm = 0.0
  ttropo_rtm = 0.0
  ems3b_clear_rtm = 0.0
  ems3b_clear_solar_rtm = 0.0
  ems3b_clear_solar_sfc_rtm = 0.0
  bt_clear_ch4_rtm_unbiased = 0.0
  bt_clear_ch5_rtm_unbiased = 0.0
  rad_clear_ch4_rtm_unbiased = 0.0
  rad_clear_ch5_rtm_unbiased = 0.0

!--- surface emissivity
   allocate(sfc_emiss_3b(num_pix,j_min:j_max),sfc_emiss_4(num_pix,j_min:j_max),sfc_emiss_5(num_pix,j_min:j_max))
   sfc_emiss_3b = 0.0
   sfc_emiss_4 = 0.0
   sfc_emiss_5 = 0.0

!------------------------------------------------------------------------
! create other arrays necessary for pixel processing
! note, h2o_coef(:,3,:) for n05-n14 are fill values as there is no ch3a
!------------------------------------------------------------------------
   h2o_tau_coef = 0.0
   h2o_tau_coef(2,1,:)  = (/  0.0000953,  0.0038987, -0.0001111/)   !Metop-A
   h2o_tau_coef(2,2,:)  = (/  0.0225902,  0.0461732, -0.0033542/)
   h2o_tau_coef(2,3,:)  = (/ -0.0000252,  0.0010506, -0.0000086/)   
   h2o_tau_coef(5,1,:)  = (/  0.0032367,  0.0130541, -0.0007250/)   !TIROS-N
   h2o_tau_coef(5,2,:)  = (/  0.0217386,  0.0428737, -0.0030683/)
   h2o_tau_coef(5,3,:)  = (/ -0.0000271,  0.0010493, -0.0000041/)   
   h2o_tau_coef(6,1,:)  = (/  0.0008346,  0.0051371, -0.0001986/)   !NOAA-6
   h2o_tau_coef(6,2,:)  = (/  0.0201284,  0.0422938, -0.0030037/)
   h2o_tau_coef(6,3,:)  = (/ -0.0000123,  0.0010351, -0.0000012/)  
   h2o_tau_coef(7,1,:)  = (/  0.00015164,  0.00427070, -0.00014814/) !NOAA-7
   h2o_tau_coef(7,2,:)  = (/  0.02228949,  0.04605545, -0.00341377/)
   h2o_tau_coef(7,3,:)  = (/ -0.00004014,  0.00108599, -0.00001179/)
   h2o_tau_coef(8,1,:)  = (/  0.0013499,  0.0060942, -0.0002900/)   !NOAA-8
   h2o_tau_coef(8,2,:)  = (/  0.0201563,  0.0427640, -0.0030115/)
   h2o_tau_coef(8,3,:)  = (/ -0.0000123,  0.0010351, -0.0000012/)  
   h2o_tau_coef(9,1,:) =  (/ 0.00032776, 0.00516792, -0.00020674/) !NOAA-9
   h2o_tau_coef(9,2,:) =  (/ 0.01976890, 0.04853170, -0.00371686/)
   h2o_tau_coef(9,3,:) =  (/-0.00000419, 0.00103691, -0.00001055/)
   h2o_tau_coef(10,1,:) = (/  0.0001511,  0.0041184, -0.0001343/)  !NOAA-10
   h2o_tau_coef(10,1,:) = (/  0.0216988,  0.0429996, -0.0030172/)
   h2o_tau_coef(10,1,:) = (/ -0.00003659, 0.00108388,-0.00001249/)
   h2o_tau_coef(11,1,:) = (/  0.00034511, 0.00507950,-0.00018612/)  !NOAA-11
   h2o_tau_coef(11,2,:) = (/  0.02171927, 0.04394474,-0.00304032/)
   h2o_tau_coef(11,3,:) = (/ -0.00002597, 0.00106648,-0.00000973/)
   h2o_tau_coef(12,1,:) = (/  0.0005350,  0.0053901, -0.0001962/)  !NOAA-12
   h2o_tau_coef(12,2,:) = (/  0.0211956,  0.0433870, -0.0030448/)
   h2o_tau_coef(12,3,:) = (/ -0.0000145,  0.0010398, -0.0000040/)
   h2o_tau_coef(14,1,:) = (/  0.00041998, 0.00572566,-0.00021681/)  !NOAA-14
   h2o_tau_coef(14,2,:) = (/  0.02276049, 0.04554781,-0.00324467/)
   h2o_tau_coef(14,3,:) = (/ -0.00002743, 0.00107200,-0.00000834/)
   h2o_tau_coef(15,1,:) = (/  0.00009828, 0.00391475,-0.00011166/)  !NOAA-15
   h2o_tau_coef(15,2,:) = (/  0.02386625, 0.04784030,-0.00355178/)
   h2o_tau_coef(15,2,:) = (/ -0.00001264, 0.00105992,-0.00000860/)
   h2o_tau_coef(16,1,:) = (/  0.00009556, 0.00408262,-0.00012258/)  !NOAA-16
   h2o_tau_coef(16,2,:) = (/  0.02415691, 0.05014109,-0.00378751/)
   h2o_tau_coef(16,3,:) = (/ -0.00000815, 0.00112846,-0.00001398/)
   h2o_tau_coef(17,1,:) = (/  0.00008434, 0.00370099,-0.00010182/)  !NOAA-17
   h2o_tau_coef(17,2,:) = (/  0.02399855, 0.04743096,-0.00347743/)
   h2o_tau_coef(17,3,:) = (/ -0.00001645, 0.00104428,-0.00000607/)
   h2o_tau_coef(18,1,:) = (/  0.00009604, 0.00371249,-0.00010250/)  !NOAA-18
   h2o_tau_coef(18,2,:) = (/  0.02563643, 0.05029506,-0.00366688/)
   h2o_tau_coef(18,3,:) = (/ -0.00001689, 0.00105943,-0.00000508/)
   h2o_tau_coef(19,1,:) = (/  0.00006699, 0.00381269,-0.00011757/)  !NOAA-19
   h2o_tau_coef(19,2,:) = (/  0.02016030, 0.04342880,-0.00322844/)
   h2o_tau_coef(19,3,:) = (/ -0.00002734, 0.00105867,-0.00001211/)

!--- fixd values based on Ignatov and Stowe
   tau_ray_corr(:,1) = 0.0568  !ch1
   tau_h2o_corr(:,1) = 0.0031
   tau_o3_corr(:,1) = 0.0342
   tau_o2_corr(:,1) = 0.0007
   tau_ch4_corr(:,1) = 0.00
   tau_co2_corr(:,1) = 0.00
   tau_ray_corr(:,2) = 0.0187  !ch2
   tau_h2o_corr(:,2) = 0.0721
   tau_o3_corr(:,2) = 0.0008
   tau_o2_corr(:,2) = 0.0137
   tau_co2_corr(:,2) = 0.00
   tau_ch4_corr(:,2) = 0.00
   tau_ray_corr(:,3) = 0.0013  !ch3a
   tau_h2o_corr(:,3) = 0.0005
   tau_o3_corr(:,3) = 0.00
   tau_o2_corr(:,3) = 0.00
   tau_co2_corr(:,3) = 0.0161
   tau_ch4_corr(:,3) = 0.0004

!--- NOAA-07
      tau_ray_corr(7,1) = 0.0585  !ch1
      tau_h2o_corr(7,1) = 0.0034
      tau_o3_corr(7,1) = 0.0334
      tau_o2_corr(7,1) = 0.0031
      tau_ch4_corr(7,1) = 0.00
      tau_co2_corr(7,1) = 0.00
      tau_ray_corr(7,2) = 0.0200  !ch2
      tau_h2o_corr(7,2) = 0.0662
      tau_o3_corr(7,2) = 0.0013
      tau_o2_corr(7,2) = 0.0147
      tau_co2_corr(7,2) = 0.00
      tau_ch4_corr(7,2) = 0.00
!--- NOAA-09
      tau_ray_corr(9,1) = 0.0571  !ch1
      tau_h2o_corr(9,1) = 0.0033
      tau_o3_corr(9,1) = 0.0342
      tau_o2_corr(9,1) = 0.0025
      tau_ch4_corr(9,1) = 0.00
      tau_co2_corr(9,1) = 0.00
      tau_ray_corr(9,2) = 0.0199  !ch2
      tau_h2o_corr(9,2) = 0.0657
      tau_o3_corr(9,2) = 0.0012
      tau_o2_corr(9,2) = 0.0151
      tau_co2_corr(9,2) = 0.00
      tau_ch4_corr(9,2) = 0.00
!--- NOAA-10
      tau_ray_corr(10,1) = 0.0590  !ch1
      tau_h2o_corr(10,1) = 0.0032
      tau_o3_corr(10,1) = 0.0342
      tau_o2_corr(10,1) = 0.0025
      tau_ch4_corr(10,1) = 0.00
      tau_co2_corr(10,1) = 0.00
      tau_ray_corr(10,2) = 0.0195  !ch2
      tau_h2o_corr(10,2) = 0.0630
      tau_o3_corr(10,2) = 0.0010
      tau_o2_corr(10,2) = 0.0162
      tau_co2_corr(10,2) = 0.00
      tau_ch4_corr(10,2) = 0.00
!--- NOAA-11
      tau_ray_corr(11,1) = 0.0570  !ch1
      tau_h2o_corr(11,1) = 0.0040
      tau_o3_corr(11,1) = 0.0315
      tau_o2_corr(11,1) = 0.0059
      tau_ch4_corr(11,1) = 0.00
      tau_co2_corr(11,1) = 0.00
      tau_ray_corr(11,2) = 0.0200  !ch2
      tau_h2o_corr(11,2) = 0.0628
      tau_o3_corr(11,2) = 0.0012
      tau_o2_corr(11,2) = 0.0158
      tau_co2_corr(11,2) = 0.00
      tau_ch4_corr(11,2) = 0.00
!--- NOAA-12
      tau_ray_corr(12,1) = 0.0560  !ch1
      tau_h2o_corr(12,1) = 0.0044
      tau_o3_corr(12,1) = 0.0312
      tau_o2_corr(12,1) = 0.0057
      tau_ch4_corr(12,1) = 0.00
      tau_co2_corr(12,1) = 0.00
      tau_ray_corr(12,2) = 0.0198  !ch2
      tau_h2o_corr(12,2) = 0.0626
      tau_o3_corr(12,2) = 0.0011
      tau_o2_corr(12,2) = 0.0122
      tau_co2_corr(12,2) = 0.00
      tau_ch4_corr(12,2) = 0.00
!--- NOAA-14
      tau_ray_corr(14,1) = 0.0555  !ch1
      tau_h2o_corr(14,1) = 0.0053
      tau_o3_corr(14,1) = 0.0306
      tau_o2_corr(14,1) = 0.0058
      tau_ch4_corr(14,1) = 0.00
      tau_co2_corr(14,1) = 0.00
      tau_ray_corr(14,2) = 0.0190  !ch2
      tau_h2o_corr(14,2) = 0.0694
      tau_o3_corr(14,2) = 0.0009
      tau_o2_corr(14,2) = 0.0152
      tau_co2_corr(14,2) = 0.00
      tau_ch4_corr(14,2) = 0.00
!--- NOAA-15
      tau_ray_corr(15,1) = 0.0565  !ch1
      tau_h2o_corr(15,1) = 0.0030
      tau_o3_corr(15,1) = 0.0341
      tau_o2_corr(15,1) = 0.0006
      tau_ch4_corr(15,1) = 0.00
      tau_co2_corr(15,1) = 0.00
      tau_ray_corr(15,2) = 0.0190  !ch2
      tau_h2o_corr(15,2) = 0.0694
      tau_o3_corr(15,2) = 0.0008
      tau_o2_corr(15,2) = 0.0137
      tau_co2_corr(15,2) = 0.00
      tau_ch4_corr(15,2) = 0.00
      tau_ray_corr(15,3) = 0.0013  !ch3a
      tau_h2o_corr(15,3) = 0.0004
      tau_o3_corr(15,3) = 0.00
      tau_o2_corr(15,3) = 0.00
      tau_co2_corr(15,3) = 0.0161
      tau_ch4_corr(15,3) = 0.0006
!---- NOAA-16
      tau_ray_corr(16,1) = 0.0568  !ch1
      tau_h2o_corr(16,1) = 0.0031
      tau_o3_corr(16,1) = 0.0342
      tau_o2_corr(16,1) = 0.0007
      tau_ch4_corr(16,1) = 0.00
      tau_co2_corr(16,1) = 0.00
      tau_ray_corr(16,2) = 0.0187  !ch2
      tau_h2o_corr(16,2) = 0.0721
      tau_o3_corr(16,2) = 0.0008
      tau_o2_corr(16,2) = 0.0137
      tau_co2_corr(16,2) = 0.00
      tau_ch4_corr(16,2) = 0.00
      tau_ray_corr(16,3) = 0.0013  !ch3a
      tau_h2o_corr(16,3) = 0.0005
      tau_o3_corr(16,3) = 0.00
      tau_o2_corr(16,3) = 0.00
      tau_co2_corr(16,3) = 0.0161
      tau_ch4_corr(16,3) = 0.0004

end subroutine CREATE_PIXEL_ARRAYS

!------------------------------------------------------------------------------
subroutine DESTROY_PIXEL_ARRAYS()

  deallocate(buffer)
  deallocate(header_buffer)
  deallocate(segment_buffer)

  deallocate(chan_counts,chan_counts_sg,scan_space_counts,scan_bb_counts, &
           cld_mask_1b,cld_mask,cld_type,temp_mask, &
           cld_test_vector,cld_test_vector_packed,cirrus_quality, &
           cld_mask_ir,cld_type_ir,cld_type_bayes,cld_type_inoue,cld_mask_qf)
  deallocate(cld_mask_bayes,posterior_cld_probability)
  deallocate(cld_phase_bayes,posterior_phase_probability)
  deallocate(max_cloud_rtm_level)
  
  deallocate(fatal,ascend,bad_scan_flag,ch3a_on,resync,  &
             clavr_status_1b,clavr_status,scan_number, &
             scan_time,scan_day,scan_year,utc_scan_time_hours, &
             pixel_local_time_hours,pixel_time)

  deallocate(ir_coef_1_1b)
  deallocate(ir_coef_2_1b)
  deallocate(ir_coef_3_1b)
  deallocate(ir_linear_slope_1b)
  deallocate(ir_linear_intercept_1b)
  deallocate(ir_linear_slope_new)
  deallocate(ir_linear_intercept_new)
  deallocate(vis_slope_1)
  deallocate(vis_slope_2)
  deallocate(vis_intercept_1)
  deallocate(vis_intercept_2)
  deallocate(vis_intersection)

  deallocate(alb1,alb2,alb3a,alb3b,rad3b,rad4,rad5)

  deallocate(pix_data)

  deallocate(land,coast,land_mask,glint_mask, &
             coast_mask,bad_pixel_mask,ch3a_on_pixel_mask,desert_mask,rad3b_ems,bt3b,&
             bt4,bt5,solzen,lat,lon,lat_1b,lon_1b,airmass,scatangle,glintzen,satzen, &
             relaz,solaz,seczen, &
             coszen,cossolzen,ems3b,volcano_mask,coast_mask_nwp,space_mask,rut,tut)
  deallocate(fire,dust,smoke,snow,snow_hires,sfc_type,zsfc)
  deallocate(sst_anal,sst_anal_err,sst_anal_cice,sst_anal_uni,temp_pix_array,temp_pix_array_2, &
             pix_array_1,pix_array_2)
  deallocate(solar_contamination_mask)
  deallocate(sst_oper,ndvi,ndsi)
  deallocate(lat_anchor_1b)
  deallocate(lon_anchor_1b)
  deallocate(solzen_anchor)
  deallocate(satzen_anchor)
  deallocate(scatangle_anchor)
  deallocate(glintzen_anchor)
  deallocate(relaz_anchor)
  deallocate(solaz_anchor)
  deallocate(sst_mask,alb21,alb31,bt45,bt3b4,bt3b5)
  deallocate(sst_masked)
  deallocate(one_byte_temp)
  deallocate(two_byte_temp)
  deallocate(one_byte_anchor_temp)
  deallocate(two_byte_anchor_temp)
  deallocate(dcc_mask)

 if (cld_flag == sym%YES) then

  !--- split-window cloud props
  deallocate(Tc_sw)
  deallocate(ec_sw)
  deallocate(tau_sw)
  deallocate(Tc_sw_error)
  deallocate(ec_sw_error)
  deallocate(beta_sw_error)
  deallocate(Tc_sw_qf)
  deallocate(ec_sw_qf)
  deallocate(beta_sw)
  deallocate(Pc_sw)
  deallocate(Zc_sw)
  deallocate(cld_layer_sw)
  deallocate(beta_sw_qf)

  !--- visible cloud props
  deallocate(tau_vis)
  deallocate(tau_vis_ap)
  deallocate(vis_ref_fm)
  deallocate(re_vis)
  deallocate(lwp_vis)
  deallocate(iwp_vis)
  deallocate(iwp_tau_vis)
  deallocate(H_vis)
  deallocate(N_vis)
  deallocate(tau_vis_qf)
  deallocate(re_vis_qf)
  deallocate(small_particle_tau_re_qf)
  deallocate(large_particle_tau_re_qf)

  deallocate(cloud_063um_albedo)
  deallocate(cloud_063um_spherical_albedo)
  deallocate(cloud_063um_transmission_view)
  deallocate(cloud_063um_transmission_solar)
  
  deallocate(cost_dcomp)
  deallocate(error_cov_matrix_cod)
  deallocate(error_cov_matrix_ref)
  deallocate(dcomp_atmos_correct_AC_VIS)
  deallocate(dcomp_atmos_correct_AC_IR)
  
  deallocate(Pc_lower_cloud)
  deallocate(Zc_lower_cloud)
  deallocate(Tc_lower_cloud)

 endif

!--- non-cloud properties
   deallocate(tsfc)
   deallocate(sst_qf)
   deallocate(ndvi_qf)
   deallocate(tsfc_qf)

!--- aerosol
  if (aer_flag == sym%YES) then
     deallocate(aot1)
     deallocate(aot2)
     deallocate(aot3a)
     deallocate(aot_qf)
  endif

!--- earth radiation budget
  if (erb_flag == sym%YES) then
     deallocate(olr)
     deallocate(olr_qf)
     deallocate(swar)
     deallocate(swar_qf)
  endif

!--- ash
  if (ash_flag == sym%YES) then
    deallocate(ash_probability)
    deallocate(ash_mass_loading)
    deallocate(ash_height)
    deallocate(ash_pressure)
    deallocate(ash_temperature)
    deallocate(ash_ir_optical_depth)
    deallocate(ash_vis_optical_depth)
    deallocate(ash_effective_radius)
    deallocate(ash_emiss)
    deallocate(ash_temperature_qf)
    deallocate(ash_emiss_qf)
    deallocate(ash_beta1112_qf)
    deallocate(ash_emiss_tot)
    deallocate(ash_beta1112)
    deallocate(ash_beta1112_tot)
    deallocate(ash_beta1112_opaque)
    deallocate(ash_beta1112_tot_multi_low)
    deallocate(ash_beta1112_tot_multi_mid)
    deallocate(ash_generic1)
    deallocate(ash_generic2)
    deallocate(ash_emissivity_error)
    deallocate(ash_temperature_error)
    deallocate(ash_beta1112_error)
    deallocate(ash_qf)
    deallocate(ash_mask)
    deallocate(ash_x_lrc)
    deallocate(ash_y_lrc)
  endif



  !--- clear-sky reflectance estimates
  deallocate(alb1_clear,alb2_clear, alb3a_clear)

  !--- atmospherically corrected parameters
  deallocate(alb1_sfc,alb2_sfc,alb3a_sfc,alb3b_sfc, &
             rad3b_sfc,rad4_sfc,rad5_sfc,ndvi_sfc,&
             bt4_sfc,rad3b_ems_sfc,ems3b_sfc,rsr,rsr_qf)

!--- modis white sky reflectances
  deallocate(alb1_sfc_white_sky)
  deallocate(alb3a_sfc_white_sky)

!--- nwp and rtm indices
  deallocate(i_nwp,j_nwp,ivza_rtm,i_nwp_x,j_nwp_x,lat_nwp_fac,lon_nwp_fac)

!--- local radiative center indices
  deallocate(i_lrc)
  deallocate(j_lrc)

!--- uniformity metrics
   deallocate &
             (bt4_mean_3x3, bt4_max_3x3,bt4_min_3x3,bt4_std_3x3, &
              alb1_mean_3x3,alb1_max_3x3,alb1_min_3x3,alb1_std_3x3, &
              bt3b_mean_3x3,bt3b_max_3x3,bt3b_min_3x3,bt3b_std_3x3, &
              zsfc_mean_3x3,zsfc_max_3x3,zsfc_min_3x3,zsfc_std_3x3, &
              ems3b_mean_3x3,ems3b_max_3x3,ems3b_min_3x3,ems3b_std_3x3,&
              i_max_bt4_3x3, j_max_bt4_3x3, &
              i_min_bt4_3x3, j_min_bt4_3x3)

   deallocate(bt45_mean_3x3,bt45_min_3x3,bt45_max_3x3,bt45_std_3x3)

   deallocate(bt45_bt4_max_3x3)

   deallocate(cloud_fraction_3x3)
   deallocate(cloud_fraction_uncer_3x3)

   deallocate (alb1_sfc_white_sky_mean_3x3)
   deallocate (alb1_sfc_white_sky_max_3x3)
   deallocate (alb1_sfc_white_sky_min_3x3)
   deallocate (alb1_sfc_white_sky_std_3x3)

   deallocate(ems3b_median_3x3)
   deallocate(ems3b_std_median_3x3)
   deallocate(bt4_median_3x3)
   deallocate(bt4_std_median_3x3)
   deallocate(bt3b_median_3x3)
   deallocate(bt3b_std_median_3x3)

!--- pixel rtm parameters
  deallocate(rad_clear_ch3b_rtm, rad_clear_ch4_rtm, rad_clear_ch5_rtm, &
             bt_clear_ch3b_rtm, bt_clear_ch4_rtm, bt_clear_ch5_rtm, &
             rad_atm_ch3b_rtm, rad_atm_ch4_rtm, rad_atm_ch5_rtm, &
             trans_atm_ch3b_rtm, trans_atm_ch4_rtm, trans_atm_ch5_rtm, &
             trans_atm_ch3b_solar_rtm, &
             trans_atm_ch3b_solar_total_rtm, &
             emiss_11_tropo_rtm,  &
             emiss_11_tropo_nadir_rtm,  &
             beta_11_12_tropo_rtm, &
             bt_clear_ch3b_solar_rtm,  &
             rad_clear_ch3b_solar_rtm, &
             tsfc_rtm,tpw_rtm,ttropo_rtm,ems3b_clear_rtm, &
             ems3b_clear_solar_rtm, ems3b_clear_solar_sfc_rtm, &
             bt_clear_ch4_rtm_unbiased, bt_clear_ch5_rtm_unbiased, &
             rad_clear_ch4_rtm_unbiased, rad_clear_ch5_rtm_unbiased)

  !--- ir cloud layer
  deallocate(beta_11_12_850mb_rtm)
  deallocate(beta_11_12_mean_ec)
  deallocate(Pc_opaque_cloud)
  deallocate(Zc_opaque_cloud)
  deallocate(Tc_opaque_cloud)

  deallocate(sfc_emiss_3b,sfc_emiss_4,sfc_emiss_5)

end subroutine DESTROY_PIXEL_ARRAYS

!-----------------------------------------------------------------------------
!  reset pixel arrays to missing
!-----------------------------------------------------------------------------
subroutine RESET_PIXEL_ARRAYS_TO_MISSING()
      lat = missing_value_real4
      lon = missing_value_real4
      lat_1b = missing_value_real4
      lon_1b = missing_value_real4
      satzen = missing_value_real4
      solzen = missing_value_real4
      coszen = missing_value_real4
      cossolzen = missing_value_real4
      relaz = missing_value_real4
      solaz = missing_value_real4
      glintzen = missing_value_real4
      alb1 = missing_value_real4
      alb2 = missing_value_real4
      alb3a = missing_value_real4
      alb3b = missing_value_real4
      ems3b = missing_value_real4
      rad3b = missing_value_real4
      rad4 = missing_value_real4
      rad5 = missing_value_real4

      dcc_mask = missing_value_int1
      bt3b  = missing_value_real4
      bt4  = missing_value_real4
      bt5  = missing_value_real4
      sst_anal = missing_value_real4
      sst_mask = missing_value_real4
      sst_oper = missing_value_real4
      sst_masked = missing_value_real4
      tsfc = missing_value_real4
      clavr_status = 0
      cld_mask = sym%PROB_CLEAR
      cld_mask_ir = sym%PROB_CLEAR
      cld_mask_bayes = sym%PROB_CLEAR
      cld_phase_bayes = sym%CLEAR_PHASE
      posterior_cld_probability = missing_value_real4
      posterior_phase_probability = missing_value_real4
      cld_type = sym%UNKNOWN_TYPE
      cld_type_ir = sym%UNKNOWN_TYPE
      cld_type_bayes = sym%UNKNOWN_TYPE
      cld_type_inoue = 0
      max_cloud_rtm_level = 0
      cirrus_quality = 1
      alb1_clear = missing_value_real4
      alb1_sfc = missing_value_real4
      alb2_sfc = missing_value_real4
      rsr = missing_value_real4
      rsr_qf = 0
      ndvi_sfc = missing_value_real4
      cld_test_vector = 0
      cld_test_vector_packed = 0
      bt4_sfc  = missing_value_real4
      pix_array_1 = missing_value_real4
      pix_array_2 = missing_value_real4

      ems3b_median_3x3 = 0.0
      ems3b_std_median_3x3 = 0.0
      bt3b_median_3x3 = 0.0
      bt3b_std_median_3x3 = 0.0
      bt4_median_3x3 = 0.0
      bt4_std_median_3x3 = 0.0
      bt4_mean_3x3 = 0.0
      bt4_min_3x3 = 0.0
      bt4_max_3x3 = 0.0
      bt4_std_3x3 = 0.0
      bt45_mean_3x3 = 0.0
      bt45_min_3x3 = 0.0
      bt45_max_3x3 = 0.0
      bt45_std_3x3 = 0.0
      bt3b_mean_3x3 = 0.0
      bt3b_min_3x3 = 0.0
      bt3b_max_3x3 = 0.0
      bt3b_std_3x3 = 0.0
      zsfc_mean_3x3 = 0.0
      zsfc_min_3x3 = 0.0
      zsfc_max_3x3 = 0.0
      zsfc_std_3x3 = 0.0
      alb1_mean_3x3 = 0.0
      alb1_min_3x3 = 0.0
      alb1_max_3x3 = 0.0
      alb1_std_3x3 = 0.0
      alb1_sfc_white_sky_mean_3x3 = 0.0
      alb1_sfc_white_sky_max_3x3 = 0.0
      alb1_sfc_white_sky_min_3x3 = 0.0
      alb1_sfc_white_sky_std_3x3 = 0.0

      bt45_bt4_max_3x3 = 0.0
      cloud_fraction_3x3 = 0.0
      cloud_fraction_uncer_3x3 = 0.0

      if (cld_flag == sym%YES) then

      Tc_sw = missing_value_real4
      ec_sw = missing_value_real4
      tau_sw = missing_value_real4
      beta_sw = missing_value_real4
      Tc_sw_error = missing_value_real4
      ec_sw_error = missing_value_real4
      beta_sw_error = missing_value_real4

      Tc_sw_qf = 0
      ec_sw_qf = 0
      beta_sw_qf = 0
      cld_layer_sw = missing_value_int1
      Pc_sw = missing_value_real4
      Zc_sw = missing_value_real4
      tau_vis = missing_value_real4
      tau_vis_ap = missing_value_real4
      vis_ref_fm = missing_value_real4
      re_vis = missing_value_real4
      lwp_vis = missing_value_real4
      iwp_vis = missing_value_real4
      iwp_tau_vis = missing_value_real4
      H_vis = missing_value_real4
      N_vis = missing_value_real4
      tau_vis_qf = 0
      re_vis_qf = 0
      small_particle_tau_re_qf = 0
      large_particle_tau_re_qf = 0

      cloud_063um_albedo = missing_value_real4
      cloud_063um_spherical_albedo = missing_value_real4
      cloud_063um_transmission_view = missing_value_real4
      cloud_063um_transmission_solar = missing_value_real4
      dcomp_atmos_correct_AC_VIS = missing_value_real4
      dcomp_atmos_correct_AC_IR = missing_value_real4
    
      endif

      emiss_11_tropo_rtm = missing_value_real4
      emiss_11_tropo_nadir_rtm = missing_value_real4
      beta_11_12_tropo_rtm = missing_value_real4
      beta_11_12_850mb_rtm = missing_value_real4
      beta_11_12_mean_ec = missing_value_real4
      Pc_opaque_cloud = missing_value_real4
      Zc_opaque_cloud = missing_value_real4
      Tc_opaque_cloud = missing_value_real4
      Pc_lower_cloud = missing_value_real4
      Zc_lower_cloud = missing_value_real4
      Tc_lower_cloud = missing_value_real4
      bt_clear_ch3b_solar_rtm = missing_value_real4
      bt_clear_ch3b_rtm = missing_value_real4
      bt_clear_ch4_rtm = missing_value_real4
      bt_clear_ch5_rtm = missing_value_real4
      rad_clear_ch3b_solar_rtm = missing_value_real4
      rad_clear_ch3b_rtm = missing_value_real4
      rad_clear_ch5_rtm = missing_value_real4
      rad_clear_ch5_rtm = missing_value_real4
      trans_atm_ch3b_rtm = missing_value_real4
      trans_atm_ch3b_solar_rtm = missing_value_real4
      trans_atm_ch3b_solar_total_rtm = missing_value_real4
      trans_atm_ch4_rtm = missing_value_real4
      trans_atm_ch5_rtm = missing_value_real4
      rad_atm_ch3b_rtm = missing_value_real4
      rad_atm_ch4_rtm = missing_value_real4
      rad_atm_ch5_rtm = missing_value_real4
      ems3b_clear_rtm = missing_value_real4
      ems3b_clear_solar_rtm = missing_value_real4
      ems3b_clear_solar_sfc_rtm = missing_value_real4
      ivza_rtm = missing_value_int1
      bt_clear_ch4_rtm_unbiased = missing_value_real4
      bt_clear_ch5_rtm_unbiased = missing_value_real4
      rad_clear_ch4_rtm_unbiased = missing_value_real4
      rad_clear_ch5_rtm_unbiased = missing_value_real4

      desert_mask = sym%NO
      sfc_type = sym%WATER_SFC
      snow = sym%NO_SNOW
      snow_hires = sym%NO_SNOW
      land = sym%DEEP_OCEAN
      land_mask = sym%NO
      glint_mask = sym%NO
      space_mask = sym%SPACE
    
      rut = 0
      tut = 0

      utc_scan_time_hours = missing_value_real4
      pixel_local_time_hours = missing_value_real4
      pixel_time = missing_value_real4

      i_lrc = missing_value_int4
      j_lrc = missing_value_int4

end subroutine RESET_PIXEL_ARRAYS_TO_MISSING


!-----------------------------------------------------------
! end of module
!-----------------------------------------------------------
end module AVHRR_PIXEL_COMMON
