!$Id: calibration_constants.f90,v 1.1.1.1 2008/10/31 19:52:02 heidinger Exp $
!-----------------------------------------------------------------------
!@***h* PATMOSX/calibration_constants.f90
! 
! NAME
!   calibration_constants.f90
! 
! LOCATION
!   /patmosx/src/calibration_constants.f90
! 
! PURPOSE
!  This module serves as a common block for passing the 
!  instrument and calibration coefficients
! 
! DESCRIPTION
!  This module serves as a common block for passing the 
!  instrument and calibration coefficients
!
! AUTHOR
!   Andrew Heidinger, NOAA/NESDIS
! 
! CREATION DATE
!   unk
!
! COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
!   THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
!   TO USERS.
!
! NOTES
!  none
! 
! EXTERNALS
!  none
!
! INPUTS_OUTPUTS
!  none
!
! MODIFICATION HISTORY
!
!
!@*****
!------------------------------------------------------------------------
! This module serves as a common block for passing the 
! instrument and calibration coefficients
!
! Author: Andrew Heidinger, NOAA/NESDIS
!
! File I/O - none
!
! Public routines: none
!
! Private routines: none
!
!----------------------------------------------------------------------
module CALIBRATION_CONSTANTS

  use CONSTANTS

  implicit none

!--- Planck Constants
  real (kind=real4), parameter, public:: c1 = 1.191062e-5, &
                                         c2 = 1.4387863

  real (kind=real4), public,save:: a1_3b,a2_3b,nu_3b,a1_4,a2_4,nu_4,a1_5,a2_5,nu_5

!----- variables read in from instrument constant files (planck stored in constants module)
 real (kind=real4), save, public:: b0_3b,b1_3b,b2_3b,b0_4,b1_4,b2_4,b0_5,b1_5,b2_5, &
                      space_rad_3b,space_rad_4,space_rad_5
 real(kind=real4), dimension(0:4,4),public,save:: prt_coef
 real(kind=real4), dimension(4),public,save:: prt_weight
 real(kind=real4),save,public:: ch1_gain_low_0,ch1_gain_high_0,ch2_gain_low_0,&
                ch2_gain_high_0,ch3a_gain_low_0,ch3a_gain_high_0, &
                ch1_degrad_low_1,ch2_degrad_low_1,ch3a_degrad_low_1, &
                ch1_degrad_high_1,ch2_degrad_high_1,ch3a_degrad_high_1, &
                ch1_degrad_low_2,ch2_degrad_low_2,ch3a_degrad_low_2, &
                ch1_degrad_high_2,ch2_degrad_high_2,ch3a_degrad_high_2, &
                ch1_dark_count,ch2_dark_count,ch3a_dark_count, &
                ch1_dark_count_cal,ch2_dark_count_cal,ch3a_dark_count_cal, &
                ch1_switch_count,ch2_switch_count, ch3a_switch_count, &
                ch1_gain_low,ch1_gain_high,ch2_gain_low,ch2_gain_high, &
                ch3a_gain_low,ch3a_gain_high, alb1_switch,alb2_switch, &
                alb3a_switch,ch1_switch_count_cal,ch2_switch_count_cal, &
                ch3a_switch_count_cal

 real(kind=real4),save,public:: sun_earth_distance, launch_date

 !--- MCSST values
 real(kind=real4),save,public:: b1_day_mask,b2_day_mask, &
                                b3_day_mask,b4_day_mask

 real(kind=real4),save,public:: solar_3b,solar_3b_nu,ew_3b
 character(len=7),save,public:: sat_name

end module CALIBRATION_CONSTANTS
