! $Id: cell_hdf_1.f90,v 1.7 2010/01/29 18:28:33 heidinger Exp $
module CELL_HDF_ROUTINES_1
!-----------------------------------------------------------------
!@***h* PATMOSX/cell_hdf_1.f90
! 
! NAME
!   cell_hdf_1.f90
! 
! LOCATION
!   /patmosx/src/cell_hdf_1.f90
! 
! PURPOSE
!   CELL_HDF_ROUTINES
! 
! AUTHOR
!   Andrew Heidinger, NOAA/NESDIS
! 
! CREATION DATE
!   unk
!
! COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
!   THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
!   TO USERS.
!
! NOTES
!  none
!
! EXTERNALS
!  Public routines used here:
!  WRITE_GRIDCELL_DATA - routine to write CLAVR-x sds's to 
!                        orbital grid file
!
! INPUTS_OUTPUTS
!  File I/O - data is written to the orbital gridcell file using HDF4
!
!
!@*****
!------------------------------------------------------------------
! CELL_HDF_ROUTINES
!
! File I/O - data is written to the orbital gridcell file using HDF4
!
! Public routines used here:
!  WRITE_GRIDCELL_DATA - routine to write CLAVR-x sds's to 
!                        orbital grid file
!
! Private routines used here - None.
!-----------------------------------------------------------------
   use CONSTANTS
   use HDF
   use HDF_PARAMS
   use LEVEL3_ROUTINES
   use SCALING_PARAMETERS

   implicit none
   private
   public:: WRITE_GRIDCELL_DATA

   integer(kind=int4), dimension(:), allocatable, private:: cell_index

   character(*), parameter, private :: MOD_PROMPT = " CELL_HDF_ROUTINES: "

contains

!------------------------------------------------------------------------------
!  Write gridded data to already opened HDF file
!------------------------------------------------------------------------------
subroutine WRITE_GRIDCELL_DATA(num_cells,num_cells_with_data,dir_cell,file_1b,grid_format_flag,dlat, &
                             c1,c2,a1_3b,a2_3b,nu_3b,a1_4,a2_4,nu_4,a1_5,a2_5,nu_5, &
                             solar_3b_nu,sun_earth_distance,therm_cal_1b,ref_cal_1b,nav_flag, &
                             use_sst_anal,sst_anal_opt,cloud_type_ir_flag,modis_clr_alb_flag,nwp_flag, &
                             ch1_gain_low,ch1_gain_high,ch1_switch_count,ch1_dark_count, &
                             ch2_gain_low,ch2_gain_high,ch2_switch_count,ch2_dark_count, &
                             ch3a_gain_low,ch3a_gain_high,ch3a_switch_count,ch3a_dark_count,&
                             start_year,end_year,start_day,end_day,start_time,end_time,comp_flag, &
                             timerr_seconds,sc_id)

 real(kind=real4), intent(in):: dlat
 integer:: i,erstat,icell
 integer(kind=int4), intent(in):: num_cells,num_cells_with_data,grid_format_flag,nwp_flag,modis_clr_alb_flag
 character(len=*), intent(in):: dir_cell,file_1b
 character(len=72):: file_cell
 real(kind=real4), intent(in):: c1,c2,a1_3b,a2_3b,nu_3b,a1_4,a2_4,nu_4,a1_5,a2_5,nu_5,solar_3b_nu,sun_earth_distance, &
                                ch1_gain_low,ch1_gain_high,ch1_switch_count,ch1_dark_count, &
                                ch2_gain_low,ch2_gain_high,ch2_switch_count,ch2_dark_count, &
                                ch3a_gain_low,ch3a_gain_high,ch3a_switch_count,ch3a_dark_count
 integer, intent(in):: therm_cal_1b, nav_flag,use_sst_anal,cloud_type_ir_flag,ref_cal_1b, sst_anal_opt, comp_flag
 integer(kind=int4), intent(in):: start_time, end_time, sc_id
 integer(kind=int2), intent(in):: start_year, end_year, start_day, end_day
 real(kind=real4), intent(in):: timerr_seconds
 character(len=11):: grid_format
 integer:: sd_id_cell, istatus
 


! HDF function declarations
 integer:: sfstart, sfend

!---------------------------------------------------------
!-- define grid-cell file structure
!---------------------------------------------------------
     file_cell = trim(file_1b)//".level3.hdf"

     print *, EXE_PROMPT, MOD_PROMPT, "creating grid-cell HDF file ", trim(file_cell)

     sd_id_cell = sfstart(trim(dir_cell)//trim(file_cell),DFACC_CREATE)

     erstat = 0
     if (sd_id_cell <  0) then
       print *, EXE_PROMPT, MOD_PROMPT, "grid-cell HDF file creation failed. Exiting..."
       erstat = 38
       stop 38
     endif

     if (grid_format_flag == 0) then
         grid_format = "EQUAL_AREA"
     else
         grid_format = "EQUAL_ANGLE"
     endif
    
!--- write attributes     
 call WRITE_CLAVRX_HDF_GLOBAL_ATTRIBUTES(sd_id_cell,"GRIDCELL",file_cell,file_1b, &
                           start_year,end_year,start_day,end_day,start_time,end_time,&
                           num_cells,num_cells_with_data,trim(grid_format),dlat, &
                           therm_cal_1b,ref_cal_1b,nav_flag,use_sst_anal,sst_anal_opt, &
                           cloud_type_ir_flag, modis_clr_alb_flag, nwp_flag, ch1_gain_low, ch1_gain_high, &
                           ch1_switch_count, ch1_dark_count, &
                           ch2_gain_low, ch2_gain_high, &
                           ch2_switch_count, ch2_dark_count, &
                           ch3a_gain_low, ch3a_gain_high, &
                           ch3a_switch_count, ch3a_dark_count, &
                           sun_earth_distance, &
                           c1, c2, a1_3b, a2_3b, nu_3b, &
                           a1_4, a2_4, nu_4, a1_5, a2_5, nu_5, &
                           solar_3b_nu,timerr_seconds,sc_id)
!--------------------------------------------------------------------------
! allocate memory
!--------------------------------------------------------------------------
 allocate(cell_index(num_cells_with_data))
 cell_index = 0

!----------- determine which cells have data in them
 i = 1
 do icell = 1, num_cells
   if (cell_pixel_count(icell) >= count_thresh) then
         cell_index(i) = icell
         i = i + 1
   endif
 enddo

 if (cell_index(num_cells_with_data) == 0) then
   print *, EXE_PROMPT, MOD_PROMPT, "Error in creating cell_index stopping ", num_cells_with_data
   erstat = 39
   stop 39
 endif
!------------------------------------------------------------------
! start writing SDS's
!------------------------------------------------------------------

!--- cell index
    call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(cell_index,kind=real4),"cell_index", &
                DFNT_INT32,NO_SCALING,0.0,0.0, "none", real(missing_value_int4), &
                "grid_cell_number", comp_flag, istatus)

!--- longitude
     call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(cell_lon(cell_index),kind=real4),"cell_longitude", &
                 DFNT_INT16,LINEAR_SCALING,min_lon,max_lon, "degree",missing_value_real4, &
                 "grid_cell_number",comp_flag, istatus)

!--- longitude
     call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(cell_lat(cell_index),kind=real4),"cell_latitude", &
                 DFNT_INT16,LINEAR_SCALING,min_lat,max_lat, "degree",missing_value_real4, &
                 "grid_cell_number",comp_flag, istatus)

!--- spacecraft id
     call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(cell_sc_id(cell_index),kind=real4),"spacecraft_id", &
                  DFNT_INT8,NO_SCALING,0.0,0.0, "none", real(missing_value_int1), &
                  "grid_cell_number",comp_flag, istatus)

!--- ascending/descending flag
     call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(cell_asc_des(cell_index),kind=real4),"asc_des_flag", &
                  DFNT_INT8,NO_SCALING,0.0,0.0, "none", real(missing_value_int1), &
                  "grid_cell_number",comp_flag, istatus)

!--- ch3a_on_flag
     call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(cell_ch3a_on(cell_index),kind=real4),"ch3a_on_flag", &
                  DFNT_INT8,NO_SCALING,0.0,0.0, "none", real(missing_value_int1), &
                  "grid_cell_number",comp_flag, istatus)

!--- orbit number
     call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(cell_orbit(cell_index),kind=real4),"orbit_number", &
                  DFNT_INT32,NO_SCALING,0.0,0.0, "none", real(missing_value_int4,kind=real4), &
                  "grid_cell_number",comp_flag, istatus)
!--- utc time
     call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(cell_utc_time(cell_index),kind=real4),"utc_time", &
                  DFNT_FLOAT32,NO_SCALING,0.0,0.0, "none", missing_value_real4, &
                  "grid_cell_number",comp_flag, istatus)

!--- jday
     call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(cell_day(cell_index),kind=real4),"jday", &
                  DFNT_INT16,NO_SCALING,0.0,0.0, "days", missing_value_real4, &
                  "grid_cell_number",comp_flag, istatus)

!--- year
     call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(cell_year(cell_index),kind=real4),"year", &
                  DFNT_INT16,NO_SCALING,0.0,0.0, "years", missing_value_real4, &
                  "grid_cell_number",comp_flag, istatus)

!--- surface type
     call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(cell_sfc_type(cell_index),kind=real4),"surface_type", &
                  DFNT_INT8,NO_SCALING,0.0,0.0, "none",real(missing_value_int1), &
                  "grid_cell_number",comp_flag, istatus)

!--- zenith angle
     call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(satzen_mean(cell_index),kind=real4),"sensor_zenith",&
                  DFNT_INT8,LINEAR_SCALING,min_ang,max_ang,"degrees",missing_value_real4, &
                  "grid_cell_number",comp_flag, istatus)

!--- solar zenith angle
     call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(solzen_mean(cell_index),kind=real4),"solar_zenith",&
                  DFNT_INT8,LINEAR_SCALING,min_ang,max_ang,"degrees",missing_value_real4, &
                  "grid_cell_number",comp_flag, istatus)

!--- relative azimuth angle
     call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(relaz_mean(cell_index),kind=real4),"relative_azimuth",&
                  DFNT_INT8,LINEAR_SCALING,min_ang,max_ang,"degrees",missing_value_real4, &
                  "grid_cell_number",comp_flag, istatus)

!--- glint zenith angle
     call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(glint_mean(cell_index),kind=real4),"glint_zenith",&
                  DFNT_INT8,LINEAR_SCALING,min_ang,max_ang,"degrees",missing_value_real4, &
                  "grid_cell_number",comp_flag, istatus)

!--- surface elevation
     call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(zsfc_mean(cell_index),kind=real4),"surface_elevation",&
                  DFNT_INT8,LINEAR_SCALING,min_z,max_z,"meter",missing_value_real4, &
                  "grid_cell_number",comp_flag, istatus)

!--- gpi235
     call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(cell_gpi_235(cell_index),kind=real4),"gpi_235",&
                  DFNT_INT8,LINEAR_SCALING,min_frac,max_frac,"none",missing_value_real4, &
                  "grid_cell_number",comp_flag, istatus)

!--- gpi255
     call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(cell_gpi_255(cell_index),kind=real4),"gpi_255",&
                  DFNT_INT8,LINEAR_SCALING,min_frac,max_frac,"none",missing_value_real4, &
                  "grid_cell_number",comp_flag, istatus)
!--- gpi275
     call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(cell_gpi_275(cell_index),kind=real4),"gpi_275",&
                  DFNT_INT8,LINEAR_SCALING,min_frac,max_frac,"none",missing_value_real4, &
                  "grid_cell_number",comp_flag, istatus)

!--- dcc fraction
     call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(cell_dcc_frac(cell_index),kind=real4),"dcc_frac",&
                  DFNT_INT8,LINEAR_SCALING,min_frac,max_frac,"none",missing_value_real4, &
                  "grid_cell_number",comp_flag, istatus)

!--- dcc mean ch1
     call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(cell_mean_dcc_ch1(cell_index),kind=real4),"dcc_ch1",&
                  DFNT_INT8,LINEAR_SCALING,min_alb1,max_alb1,"none",missing_value_real4, &
                  "grid_cell_number",comp_flag, istatus)

!--- dcc std dev ch1
     call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(cell_std_dcc_ch1(cell_index),kind=real4),"dcc_ch1_std",&
                  DFNT_INT8,LINEAR_SCALING,min_alb1_std,max_alb1_std,"none",missing_value_real4, &
                  "grid_cell_number",comp_flag, istatus)

!--- dust_pixel_frac
     call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(dust_frac(cell_index),kind=real4),"dust_pixel_frac",&
                  DFNT_INT8,LINEAR_SCALING,min_frac,max_frac,"none",missing_value_real4, &
                  "grid_cell_number",comp_flag, istatus)

!--- smoke_pixel_frac
     call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(smoke_frac(cell_index),kind=real4),"smoke_pixel_frac",&
                  DFNT_INT8,LINEAR_SCALING,min_frac,max_frac,"none",missing_value_real4, &
                  "grid_cell_number",comp_flag, istatus)

!--- pixel count
  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(cell_pixel_count(cell_index),kind=real4),"pixel_count",&
                  DFNT_INT16,NO_SCALING,0.0,0.0,"none",missing_value_real4, &
                  "grid_cell_number",comp_flag, istatus) 

!--- clear pixel count
  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(cell_mask_count(1,0,cell_index),kind=real4),"clr_pixel_count",&
                  DFNT_INT16,NO_SCALING,0.0,0.0,"none",missing_value_real4, &
                  "grid_cell_number",comp_flag, istatus) 

!--- probably clear pixel count
! call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(cell_mask_count(1,1,cell_index),kind=real4),"prob_clr_pixel_count",&
!                 DFNT_INT16,NO_SCALING,0.0,0.0,"none",missing_value_real4, &
!                 "grid_cell_number",comp_flag, istatus) 

!--- probably cloudy pixel count
! call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(cell_mask_count(1,2,cell_index),kind=real4),"prob_cldy_pixel_count",&
!                 DFNT_INT16,NO_SCALING,0.0,0.0,"none",missing_value_real4, &
!                 "grid_cell_number",comp_flag, istatus) 

!--- cloudy pixel count
! call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(cell_mask_count(1,3,cell_index),kind=real4),"cldy_pixel_count",&
!                 DFNT_INT16,NO_SCALING,0.0,0.0,"none",missing_value_real4, &
!                 "grid_cell_number",comp_flag, istatus) 

!--- probably cloudy water pixel count
! call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(cell_mask_count(1,2,cell_index),kind=real4),"prob_cldy_water_pixel_count",&
!                 DFNT_INT16,NO_SCALING,0.0,0.0,"none",missing_value_real4, &
!                 "grid_cell_number",comp_flag, istatus) 

!--- terminator fraction
  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(term_frac(cell_index),kind=real4), &
                  "terminator_pixel_fraction",&
                  DFNT_INT8,LINEAR_SCALING,min_frac,max_frac,"none",missing_value_real4, &
                  "grid_cell_number",comp_flag, istatus) 

!--- snow pixel fraction
  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(snow_frac(cell_index),kind=real4), &
                  "snow_pixel_fraction",&
                  DFNT_INT8,LINEAR_SCALING,min_frac,max_frac,"none",missing_value_real4, &
                  "grid_cell_number",comp_flag, istatus) 

!--- ch1 clear
  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(cell_mean(1,0,cell_index),kind=real4), &
                  "ch1_clr",&
                  DFNT_INT16,LINEAR_SCALING,min_alb1,max_alb1,"%",missing_value_real4, &
                  "grid_cell_number",comp_flag, istatus) 

!--- ch1 probably clear
! call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(cell_mean(1,1,cell_index),kind=real4), &
!                 "ch1_prob_clr",&
!                 DFNT_INT16,LINEAR_SCALING,min_alb1,max_alb1,"%",missing_value_real4, &
!                 "grid_cell_number",comp_flag, istatus) 

!--- ch1 probably cloudy
! call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(cell_mean(1,2,cell_index),kind=real4), &
!                 "ch1_prob_cldy",&
!                 DFNT_INT16,LINEAR_SCALING,min_alb1,max_alb1,"%",missing_value_real4, &
!                 "grid_cell_number",comp_flag, istatus) 

!--- ch1 cloudy
  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(cell_mean(1,3,cell_index),kind=real4), &
                  "ch1_cldy",&
                  DFNT_INT16,LINEAR_SCALING,min_alb1,max_alb1,"%",missing_value_real4, &
                  "grid_cell_number",comp_flag, istatus) 

!--- ch1
  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(cell_mean(1,4,cell_index),kind=real4), &
                  "ch1",&
                  DFNT_INT16,LINEAR_SCALING,min_alb1,max_alb1,"%",missing_value_real4, &
                  "grid_cell_number",comp_flag, istatus) 

!--- ch2 clear
  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(cell_mean(2,0,cell_index),kind=real4), &
                  "ch2_clr",&
                  DFNT_INT16,LINEAR_SCALING,min_alb2,max_alb2,"%",missing_value_real4, &
                  "grid_cell_number",comp_flag, istatus) 

!--- ch2 cloudy
  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(cell_mean(2,3,cell_index),kind=real4), &
                  "ch2_cldy",&
                  DFNT_INT16,LINEAR_SCALING,min_alb2,max_alb2,"%",missing_value_real4, &
                  "grid_cell_number",comp_flag, istatus) 

!--- ch2
! call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(cell_mean(2,4,cell_index),kind=real4), &
!                 "ch2",&
!                 DFNT_INT16,LINEAR_SCALING,min_alb2,max_alb2,"%",missing_value_real4, &
!                 "grid_cell_number",comp_flag, istatus) 


!--- ch3ab ref clear
  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(cell_mean(3,0,cell_index),kind=real4), &
                  "ch3ab_ref_clr",&
                  DFNT_INT16,LINEAR_SCALING,min_alb3a,max_alb3a,"%",missing_value_real4, &
                  "grid_cell_number",comp_flag, istatus) 

!--- ch3ab ref cloudy
! call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(cell_mean(3,3,cell_index),kind=real4), &
!                 "ch3ab_ref_cldy",&
!                 DFNT_INT16,LINEAR_SCALING,min_alb3a,max_alb3a,"%",missing_value_real4, &
!                 "grid_cell_number",comp_flag, istatus) 

!--- ch3ab ref
! call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(cell_mean(3,4,cell_index),kind=real4), &
!                 "ch3ab_ref",&
!                 DFNT_INT16,LINEAR_SCALING,min_alb3a,max_alb3a,"%",missing_value_real4, &
!                 "grid_cell_number",comp_flag, istatus) 

!--- ch3b clear
! call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(cell_mean(4,0,cell_index),kind=real4), &
!                 "ch3b_clr",&
!                 DFNT_INT16,LINEAR_SCALING,min_rad3b,max_rad3b,"mW/m^2/sr/cm^-1",missing_value_real4, &
!                 "grid_cell_number",comp_flag, istatus) 

!--- ch3b cloudy
! call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(cell_mean(4,3,cell_index),kind=real4), &
!                 "ch3b_cldy",&
!                 DFNT_INT16,LINEAR_SCALING,min_rad3b,max_rad3b,"mW/m^2/sr/cm^-1",missing_value_real4, &
!                 "grid_cell_number",comp_flag, istatus) 

!--- ch3b
! call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(cell_mean(4,4,cell_index),kind=real4), &
!                 "ch3b",&
!                 DFNT_INT16,LINEAR_SCALING,min_rad3b,max_rad3b,"mW/m^2/sr/cm^-1",missing_value_real4, &
!                 "grid_cell_number",comp_flag, istatus) 

!--- ch4 clear
  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(cell_mean(5,0,cell_index),kind=real4), &
                  "ch4_clr",&
                  DFNT_INT16,LINEAR_SCALING,min_rad4,max_rad4,"mW/m^2/sr/cm^-1",missing_value_real4, &
                  "grid_cell_number",comp_flag, istatus) 

!--- ch4 probably clear
! call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(cell_mean(5,1,cell_index),kind=real4), &
!                 "ch4_prob_clr",&
!                 DFNT_INT16,LINEAR_SCALING,min_rad4,max_rad4,"mW/m^2/sr/cm^-1",missing_value_real4, &
!                 "grid_cell_number",comp_flag, istatus) 

!--- ch4 probably cloudy
! call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(cell_mean(5,2,cell_index),kind=real4), &
!                 "ch4_prob_cldy",&
!                 DFNT_INT16,LINEAR_SCALING,min_rad4,max_rad4,"mW/m^2/sr/cm^-1",missing_value_real4, &
!                 "grid_cell_number",comp_flag, istatus) 

!--- ch4 cloudy
  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(cell_mean(5,3,cell_index),kind=real4), &
                  "ch4_cldy",&
                  DFNT_INT16,LINEAR_SCALING,min_rad4,max_rad4,"mW/m^2/sr/cm^-1",missing_value_real4, &
                  "grid_cell_number",comp_flag, istatus) 

!--- ch4 
  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(cell_mean(5,4,cell_index),kind=real4), &
                  "ch4",&
                  DFNT_INT16,LINEAR_SCALING,min_rad4,max_rad4,"mW/m^2/sr/cm^-1",missing_value_real4, &
                  "grid_cell_number",comp_flag, istatus) 


!--- ch5 clear
! call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(cell_mean(6,0,cell_index),kind=real4), &
!                 "ch5_clr",&
!                 DFNT_INT16,LINEAR_SCALING,min_rad5,max_rad5,"mW/m^2/sr/cm^-1",missing_value_real4, &
!                 "grid_cell_number",comp_flag, istatus) 

!--- ch5 cloudy
! call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(cell_mean(6,3,cell_index),kind=real4), &
!                 "ch5_cldy",&
!                 DFNT_INT16,LINEAR_SCALING,min_rad5,max_rad5,"mW/m^2/sr/cm^-1",missing_value_real4, &
!                 "grid_cell_number",comp_flag, istatus) 

!--- ch5
! call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(cell_mean(6,4,cell_index),kind=real4), &
!                 "ch5",&
!                 DFNT_INT16,LINEAR_SCALING,min_rad5,max_rad5,"mW/m^2/sr/cm^-1",missing_value_real4, &
!                 "grid_cell_number",comp_flag, istatus) 

!---------------------------------------------------------------------------------------
! standard deviations of observations
!---------------------------------------------------------------------------------------
!--- ch1 clr  std
  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(cell_std(1,0,cell_index),kind=real4), &
                  "ch1_std_clr",&
                  DFNT_INT8,LINEAR_SCALING,min_alb1_std,max_alb1_std,"%",missing_value_real4, &
                  "grid_cell_number", comp_flag, istatus) 

!--- ch1 prob clr  std
! call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(cell_std(1,1,cell_index),kind=real4), &
!                 "ch1_std_prob_clr",&
!                 DFNT_INT8,LINEAR_SCALING,min_alb1_std,max_alb1_std,"%",missing_value_real4, &
!                 "grid_cell_number", comp_flag, istatus) 

!--- ch1 prob cldy std
! call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(cell_std(1,2,cell_index),kind=real4), &
!                 "ch1_std_prob_cldy",&
!                 DFNT_INT8,LINEAR_SCALING,min_alb1_std,max_alb1_std,"%",missing_value_real4, &
!                 "grid_cell_number", comp_flag, istatus) 

!--- ch1 cldy std
  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(cell_std(1,3,cell_index),kind=real4), &
                  "ch1_std_cldy",&
                  DFNT_INT8,LINEAR_SCALING,min_alb1_std,max_alb1_std,"%",missing_value_real4, &
                  "grid_cell_number", comp_flag, istatus) 

!--- ch1 std
  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(cell_std(1,4,cell_index),kind=real4), &
                  "ch1_std",&
                  DFNT_INT8,LINEAR_SCALING,min_alb1_std,max_alb1_std,"%",missing_value_real4, &
                  "grid_cell_number", comp_flag, istatus) 

!--- ch4 clear std
  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(cell_std(5,0,cell_index),kind=real4), &
                  "ch4_std_clr",&
                  DFNT_INT8,LINEAR_SCALING,min_rad4_std,max_rad4_std,"mW/m^2/sr/cm^-1",missing_value_real4, &
                  "grid_cell_number", comp_flag, istatus) 

!--- ch4 probably clear std
! call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(cell_std(5,1,cell_index),kind=real4), &
!                 "ch4_std_prob_clr",&
!                 DFNT_INT8,LINEAR_SCALING,min_rad4_std,max_rad4_std,"mW/m^2/sr/cm^-1",missing_value_real4, &
!                 "grid_cell_number", comp_flag, istatus) 

!--- ch4 probably cloudy std
! call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(cell_std(5,2,cell_index),kind=real4), &
!                 "ch4_std_prob_cldy",&
!                 DFNT_INT8,LINEAR_SCALING,min_rad4_std,max_rad4_std,"mW/m^2/sr/cm^-1",missing_value_real4, &
!                 "grid_cell_number", comp_flag, istatus) 

!--- ch4 cloudy std
  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(cell_std(5,3,cell_index),kind=real4), &
                  "ch4_std_cldy",&
                  DFNT_INT8,LINEAR_SCALING,min_rad4_std,max_rad4_std,"mW/m^2/sr/cm^-1",missing_value_real4, &
                  "grid_cell_number",comp_flag, istatus) 

!--- ch4 std
  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(cell_std(5,4,cell_index),kind=real4), &
                  "ch4_std",&
                  DFNT_INT8,LINEAR_SCALING,min_rad4_std,max_rad4_std,"mW/m^2/sr/cm^-1",missing_value_real4, &
                  "grid_cell_number",comp_flag, istatus) 

!-------------------------------------------------------------------------------------------------
! Instrument Counts used for reflectance calibration studies - only in PATMOS-x version
!--------------------------------------------------------------------------------------------------
!--- ch1
   call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(cell_count_mean(1,cell_index),kind=real4), &
                  "ch1_count",&
                  DFNT_INT16,NO_SCALING,0.0,0.0,"none",missing_value_real4, &
                  "grid_cell_number",comp_flag, istatus)

!--- ch2
  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(cell_count_mean(2,cell_index),kind=real4), &
                  "ch2_count",&
                  DFNT_FLOAT32,NO_SCALING,0.0,0.0,"none",missing_value_real4, &
                  "grid_cell_number",comp_flag, istatus)

!--- ch3ab
  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(cell_count_mean(3,cell_index),kind=real4), &
                  "ch3_count",&
                  DFNT_FLOAT32,NO_SCALING,0.0,0.0,"none",missing_value_real4, &
                  "grid_cell_number",comp_flag, istatus)

!--- ch4
  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(cell_count_mean(4,cell_index),kind=real4), &
                  "ch4_count",&
                  DFNT_FLOAT32,NO_SCALING,0.0,0.0,"none",missing_value_real4, &
                  "grid_cell_number",comp_flag, istatus)

!--- ch5
  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(cell_count_mean(5,cell_index),kind=real4), &
                  "ch5_count",&
                  DFNT_FLOAT32,NO_SCALING,0.0,0.0,"none",missing_value_real4, &
                  "grid_cell_number",comp_flag, istatus)

!--- ch1 space count
  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(cell_space_counts(1,cell_index),kind=real4), &
                  "ch1_space_count",&
                  DFNT_FLOAT32,NO_SCALING,0.0,0.0,"none",missing_value_real4, &
                  "grid_cell_number",comp_flag, istatus)

!--- ch2 space count
  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(cell_space_counts(2,cell_index),kind=real4), &
                  "ch2_space_count",&
                  DFNT_FLOAT32,NO_SCALING,0.0,0.0,"none",missing_value_real4, &
                  "grid_cell_number",comp_flag, istatus)

!--- ch3 space count
  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(cell_space_counts(3,cell_index),kind=real4), &
                  "ch3_space_count",&
                  DFNT_FLOAT32,NO_SCALING,0.0,0.0,"none",missing_value_real4, &
                  "grid_cell_number",comp_flag, istatus)

!--- ch4 space count
  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(cell_space_counts(4,cell_index),kind=real4), &
                  "ch4_space_count",&
                  DFNT_FLOAT32,NO_SCALING,0.0,0.0,"none",missing_value_real4, &
                  "grid_cell_number",comp_flag, istatus)

!--- ch5 space count
  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(cell_space_counts(5,cell_index),kind=real4), &
                  "ch5_space_count",&
                  DFNT_FLOAT32,NO_SCALING,0.0,0.0,"none",missing_value_real4, &
                  "grid_cell_number",comp_flag, istatus)

!--------------------------------------------------------------------------------------------------
! Parameters not observed by the AVHRR
!--------------------------------------------------------------------------------------------------
!--- tsfc nwp
  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(cell_tsfc_nwp(cell_index),kind=real4), &
                  "tsfc_nwp",&
                  DFNT_INT8,LINEAR_SCALING,min_tsfc,max_tsfc,"K",missing_value_real4, &
                  "grid_cell_number",comp_flag, istatus)

!--- tair nwp
  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(cell_tair_nwp(cell_index),kind=real4), &
                  "tair_nwp",&
                  DFNT_INT8,LINEAR_SCALING,min_tsfc,max_tsfc,"K",missing_value_real4, &
                  "grid_cell_number",comp_flag, istatus)

!--- tpw nwp
  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(cell_tpw_nwp(cell_index),kind=real4), &
                  "tpw_nwp",&
                  DFNT_INT8,LINEAR_SCALING,min_tpw,max_tpw,"cm",missing_value_real4, &
                  "grid_cell_number",comp_flag, istatus)

!--- weasd nwp
  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(cell_weasd_nwp(cell_index),kind=real4), &
                  "weasd_nwp",&
                  DFNT_INT8,LINEAR_SCALING,min_weasd,max_weasd,"kg/m^2",missing_value_real4, &
                  "grid_cell_number",comp_flag, istatus)

!--- uth
  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(cell_uth_nwp(cell_index),kind=real4), &
                  "uth_nwp",&
                  DFNT_INT8,LINEAR_SCALING,min_rh,max_rh,"%",missing_value_real4, &
                  "grid_cell_number",comp_flag, istatus)

!--- tropopause temp
  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(cell_ttrop_nwp(cell_index),kind=real4), &
                  "ttrop_nwp",&
                  DFNT_INT8,LINEAR_SCALING,min_ttrop,max_ttrop,"K",missing_value_real4, &
                  "grid_cell_number",comp_flag, istatus)

!--- 500 mb heights
  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(cell_hght500_nwp(cell_index),kind=real4), &
                  "hght500_nwp",&
                  DFNT_INT8,LINEAR_SCALING,min_hght500,max_hght500,"m",missing_value_real4, &
                  "grid_cell_number",comp_flag, istatus)

!--- wind speed at 10m
  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(cell_wnd_spd_10m_nwp(cell_index),kind=real4), &
                  "wnd_spd_10m_nwp",&
                  DFNT_INT8,LINEAR_SCALING,min_wndspd,max_wndspd,"m^2",missing_value_real4, &
                  "grid_cell_number",comp_flag, istatus)

!--- wind direction at 10m
  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(cell_wnd_dir_10m_nwp(cell_index),kind=real4), &
                  "wnd_dir_10m_nwp",&
                  DFNT_INT8,LINEAR_SCALING,min_wnddir,max_wnddir,"degrees",missing_value_real4, &
                  "grid_cell_number",comp_flag, istatus)

!--- ch4 clr rtm
  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(ch4_rad_clr_nwp_mean(cell_index),kind=real4), &
                  "ch4_clr_nwp",&
                  DFNT_INT8,LINEAR_SCALING,min_rad4,max_rad4,"mW/m^2/sr/cm^-1",missing_value_real4, &
                  "grid_cell_number",comp_flag, istatus)

!--- ch5 clr rtm
! call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(ch5_rad_clr_nwp_mean(cell_index),kind=real4), &
!                 "ch5_clr_nwp",&
!                 DFNT_INT8,LINEAR_SCALING,min_rad5,max_rad5,"mW/m^2/sr/cm^-1",missing_value_real4, &
!                  "grid_cell_number",comp_flag, istatus)

!--- nwp lwp
  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(cell_lwp_nwp(cell_index),kind=real4), &
                  "lwp_nwp",&
                  DFNT_INT8,LINEAR_SCALING,min_lwp,max_lwp,"g/cm^-2",missing_value_real4, &
                  "grid_cell_number",comp_flag, istatus)

!-------------------------------------------------------------------------
! atmospherically corrected observations
!-------------------------------------------------------------------------
! call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(alb1_sfc_mean(cell_index),kind=real4), &
!                 "ch1_sfc",&
!                 DFNT_INT8,LINEAR_SCALING,min_alb1,max_alb1,"%",missing_value_real4, &
!                 "grid_cell_number",  comp_flag, istatus)

! call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(alb2_sfc_mean(cell_index),kind=real4), &
!                 "ch2_sfc",&
!                 DFNT_INT8,LINEAR_SCALING,min_alb2,max_alb2,"%",missing_value_real4, &
!                 "grid_cell_number",  comp_flag, istatus)

! call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(cell_mean_rsr(cell_index),kind=real4), &
!                 "rsr_clr",&
!                 DFNT_INT8,LINEAR_SCALING,min_rsr,max_rsr,"%",missing_value_real4, &
!                 "grid_cell_number",  comp_flag, istatus)

! call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(cell_std_rsr(cell_index),kind=real4), &
!                 "rsr_std",&
!                 DFNT_INT16,LINEAR_SCALING,min_rsr_std,max_rsr_std,"%",missing_value_real4, &
!                 "grid_cell_number",  comp_flag, istatus)

! call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(cell_dark_obs(1,cell_index),kind=real4), &
!                 "ch1_dark",&
!                 DFNT_INT8,LINEAR_SCALING,min_alb1,max_alb1,"%",missing_value_real4, &
!                 "grid_cell_number",  comp_flag, istatus)

! call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(cell_dark_obs(5,cell_index),kind=real4), &
!                 "ch4_warm",&
!                 DFNT_INT8,LINEAR_SCALING,min_rad4,max_rad4,"mW/m^2/sr/cm^-1",missing_value_real4, &
!                 "grid_cell_number",  comp_flag, istatus)

  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(alb1_clr_sfc_mean(cell_index),kind=real4), &
                  "ch1_clr_sfc",&
                  DFNT_INT8,LINEAR_SCALING,min_alb1,max_alb1,"%",missing_value_real4, &
                  "grid_cell_number",  comp_flag, istatus)

  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(alb2_clr_sfc_mean(cell_index),kind=real4), &
                  "ch2_clr_sfc",&
                  DFNT_INT8,LINEAR_SCALING,min_alb2,max_alb2,"%",missing_value_real4, &
                  "grid_cell_number",  comp_flag, istatus)

! call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(alb3a_clr_sfc_mean(cell_index),kind=real4), &
!                 "ch3a_clr_sfc",&
!                 DFNT_INT8,LINEAR_SCALING,min_alb3a,max_alb3a,"%",missing_value_real4, &
!                 "grid_cell_number",  comp_flag, istatus)

  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(alb3b_clr_sfc_mean(cell_index),kind=real4), &
                  "ch3b_ref_clr_sfc",&
                  DFNT_INT8,LINEAR_SCALING,min_alb3b,max_alb3b,"%",missing_value_real4, &
                  "grid_cell_number",  comp_flag, istatus)

! call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(rad3b_sfc_mean(cell_index),kind=real4), &
!                 "ch3b_sfc",&
!                 DFNT_INT8,LINEAR_SCALING,min_rad3b,max_rad3b,"mW/m^2/sr/cm^-1",missing_value_real4, &
!                 "grid_cell_number",  comp_flag, istatus)

! call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(rad4_sfc_mean(cell_index),kind=real4), &
!                 "ch4_sfc",&
!                 DFNT_INT8,LINEAR_SCALING,min_rad4,max_rad4,"mW/m^2/sr/cm^-1",missing_value_real4, &
!                 "grid_cell_number",  comp_flag, istatus)

! call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(rad5_sfc_mean(cell_index),kind=real4), &
!                 "ch5_sfc",&
!                 DFNT_INT8,LINEAR_SCALING,min_rad5,max_rad5,"mW/m^2/sr/cm^-1",missing_value_real4, &
!                 "grid_cell_number",  comp_flag, istatus)

! call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(rad3b_clr_sfc_mean(cell_index),kind=real4), &
!                 "ch3b_clr_sfc",&
!                 DFNT_INT8,LINEAR_SCALING,min_rad3b,max_rad3b,"mW/m^2/sr/cm^-1",missing_value_real4, &
!                 "grid_cell_number",  comp_flag, istatus)

! call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(rad4_clr_sfc_mean(cell_index),kind=real4), &
!                 "ch4_clr_sfc",&
!                 DFNT_INT8,LINEAR_SCALING,min_rad4,max_rad4,"mW/m^2/sr/cm^-1",missing_value_real4, &
!                 "grid_cell_number",  comp_flag, istatus)

! call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(rad5_clr_sfc_mean(cell_index),kind=real4), &
!                 "ch5_clr_sfc",&
!                 DFNT_INT8,LINEAR_SCALING,min_rad5,max_rad5,"mW/m^2/sr/cm^-1",missing_value_real4, &
!                 "grid_cell_number",  comp_flag, istatus)

!-------------------------------------------------------------------------
! non-cloud products
!-------------------------------------------------------------------------
  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(cell_mean_tsfc(cell_index),kind=real4), &
                  "tsfc_clr",&
                  DFNT_INT16,LINEAR_SCALING,min_tsfc,max_tsfc,"K",missing_value_real4, &
                  "grid_cell_number",  comp_flag, istatus)

  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(cell_std_tsfc(cell_index),kind=real4), &
                  "tsfc_std",&
                  DFNT_INT16,LINEAR_SCALING,min_tsfc_std,max_tsfc_std,"K",missing_value_real4, &
                  "grid_cell_number",  comp_flag, istatus)

  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(cell_sst_back(cell_index),kind=real4), &
                  "sst_back",&
                  DFNT_INT16,LINEAR_SCALING,min_sst,max_sst,"K",missing_value_real4, &
                  "grid_cell_number",  comp_flag, istatus)

  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(cell_mean_ndvi_sfc(cell_index),kind=real4), &
                  "ndvi_clr_sfc",&
                  DFNT_INT8,LINEAR_SCALING,min_ndvi,max_ndvi,"none",missing_value_real4, &
                  "grid_cell_number",  comp_flag, istatus)

  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(aot1_mean(cell_index),kind=real4), &
                  "aot1",&
                  DFNT_INT8,SQUARE_ROOT_SCALING,min_aot,max_aot,"none",missing_value_real4, &
                  "grid_cell_number",  comp_flag, istatus)

  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(aot2_mean(cell_index),kind=real4), &
                  "aot2",&
                  DFNT_INT8,SQUARE_ROOT_SCALING,min_aot,max_aot,"none",missing_value_real4, &
                  "grid_cell_number",  comp_flag, istatus)

  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(aot1_std(cell_index),kind=real4), &
                  "aot1_std",&
                  DFNT_INT8,SQUARE_ROOT_SCALING,min_aot_std,max_aot_std,"none",missing_value_real4, &
                  "grid_cell_number",  comp_flag, istatus)

  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(olr_mean(cell_index),kind=real4), &
                  "olr",&
                  DFNT_INT8,LINEAR_SCALING,min_olr,max_olr,"W/m^2",missing_value_real4, &
                  "grid_cell_number",  comp_flag, istatus)

  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(olr_std(cell_index),kind=real4), &
                  "olr_std",&
                  DFNT_INT8,LINEAR_SCALING,min_olr_std,max_olr_std,"W/m^2",missing_value_real4, &
                  "grid_cell_number",  comp_flag, istatus)

  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(olr_clr_mean(cell_index),kind=real4), &
                  "olr_clr",&
                  DFNT_INT8,LINEAR_SCALING,min_olr,max_olr,"W/m^2",missing_value_real4, &
                  "grid_cell_number",  comp_flag, istatus)

  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(olr_clr_std(cell_index),kind=real4), &
                  "olr_clr_std",&
                  DFNT_INT8,LINEAR_SCALING,min_olr_std,max_olr_std,"W/m^2",missing_value_real4, &
                  "grid_cell_number",  comp_flag, istatus)

  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(olr_cldy_mean(cell_index),kind=real4), &
                  "olr_cldy",&
                  DFNT_INT8,LINEAR_SCALING,min_olr,max_olr,"W/m^2",missing_value_real4, &
                  "grid_cell_number",  comp_flag, istatus)

  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(olr_cldy_std(cell_index),kind=real4), &
                  "olr_cldy_std",&
                  DFNT_INT8,LINEAR_SCALING,min_olr_std,max_olr_std,"W/m^2",missing_value_real4, &
                  "grid_cell_number",  comp_flag, istatus)

  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(ems3b_mean(cell_index),kind=real4), &
                  "ems3b",&
                  DFNT_INT8,LINEAR_SCALING,min_ems3b,max_ems3b,"none",missing_value_real4, &
                  "grid_cell_number",  comp_flag, istatus)

  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(ems3b_clr_mean(cell_index),kind=real4), &
                  "ems3b_clr",&
                  DFNT_INT8,LINEAR_SCALING,min_ems3b,max_ems3b,"none",missing_value_real4, &
                  "grid_cell_number",  comp_flag, istatus)

  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(ems3b_sfc_mean(cell_index),kind=real4), &
                  "ems3b_sfc",&
                  DFNT_INT8,LINEAR_SCALING,min_ems3b,max_ems3b,"none",missing_value_real4, &
                  "grid_cell_number",  comp_flag, istatus)

  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(ems3b_clr_sfc_mean(cell_index),kind=real4), &
                  "ems3b_clr_sfc",&
                  DFNT_INT8,LINEAR_SCALING,min_ems3b,max_ems3b,"none",missing_value_real4, &
                  "grid_cell_number",  comp_flag, istatus)

!--------------------------------------------------------------------------
! cloud products
!--------------------------------------------------------------------------

!--- multilayer cloud fractions
  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(frac_multilayer(cell_index),kind=real4), &
                  "frac_multilayer_cld",&
                  DFNT_INT8,LINEAR_SCALING,min_frac,max_frac,"none",missing_value_real4, &
                  "grid_cell_number",  comp_flag, istatus)

!--- cloud fractions
  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(frac_all(cell_index),kind=real4), &
                  "frac_total_cld",&
                  DFNT_INT8,LINEAR_SCALING,min_frac,max_frac,"none",missing_value_real4, &
                  "grid_cell_number",  comp_flag, istatus)

  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(frac_all_ir(cell_index),kind=real4), &
                  "frac_total_cld_ir",&
                  DFNT_INT8,LINEAR_SCALING,min_frac,max_frac,"none",missing_value_real4, &
                  "grid_cell_number",  comp_flag, istatus)

  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(frac_water(cell_index),kind=real4), &
                  "frac_water_cld",&
                  DFNT_INT8,LINEAR_SCALING,min_frac,max_frac,"none",missing_value_real4, &
                  "grid_cell_number",  comp_flag, istatus)

  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(frac_ice(cell_index),kind=real4), &
                  "frac_ice_cld",&
                  DFNT_INT8,LINEAR_SCALING,min_frac,max_frac,"none",missing_value_real4, &
                  "grid_cell_number",  comp_flag, istatus)

!--- layered cloud amounts from split window
  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(frac_high_sw(cell_index),kind=real4), &
                  "frac_high_cld_sw",&
                  DFNT_INT8,LINEAR_SCALING,min_frac,max_frac,"none",missing_value_real4, &
                  "grid_cell_number",  comp_flag, istatus)

  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(frac_mid_sw(cell_index),kind=real4), &
                  "frac_mid_cld_sw",&
                  DFNT_INT8,LINEAR_SCALING,min_frac,max_frac,"none",missing_value_real4, &
                  "grid_cell_number",  comp_flag, istatus)

  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(frac_low_sw(cell_index),kind=real4), &
                  "frac_low_cld_sw",&
                  DFNT_INT8,LINEAR_SCALING,min_frac,max_frac,"none",missing_value_real4, &
                  "grid_cell_number",  comp_flag, istatus)

!---- cloud macrophysical parameters

  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(Tc_sw_mean(cell_index),kind=real4), &
                  "cld_temp_sw",&
                  DFNT_INT8,LINEAR_SCALING,min_tc,max_tc,"K",missing_value_real4, &
                  "grid_cell_number",  comp_flag, istatus)

  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(Tc_sw_mean_wat(cell_index),kind=real4), &
                  "cld_temp_sw_wat",&
                  DFNT_INT8,LINEAR_SCALING,min_tc,max_tc,"K",missing_value_real4, &
                  "grid_cell_number",  comp_flag, istatus)

  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(Tc_sw_mean_ice(cell_index),kind=real4), &
                  "cld_temp_sw_ice",&
                  DFNT_INT8,LINEAR_SCALING,min_tc,max_tc,"K",missing_value_real4, &
                  "grid_cell_number",  comp_flag, istatus)

  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(Tc_sw_std(cell_index),kind=real4), &
                  "cld_temp_sw_std",&
                  DFNT_INT8,LINEAR_SCALING,min_tc_std,max_tc_std,"K",missing_value_real4, &
                  "grid_cell_number",  comp_flag, istatus)

  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(Tc_sw_std_wat(cell_index),kind=real4), &
                  "cld_temp_sw_std_wat",&
                  DFNT_INT8,LINEAR_SCALING,min_tc_std,max_tc_std,"K",missing_value_real4, &
                  "grid_cell_number",  comp_flag, istatus)

  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(Tc_sw_std_ice(cell_index),kind=real4), &
                  "cld_temp_sw_std_ice",&
                  DFNT_INT8,LINEAR_SCALING,min_tc_std,max_tc_std,"K",missing_value_real4, &
                  "grid_cell_number",  comp_flag, istatus)

  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(ec_sw_mean(cell_index),kind=real4), &
                  "cld_emiss_sw",&
                  DFNT_INT8,LINEAR_SCALING,min_ec,max_ec,"none",missing_value_real4, &
                  "grid_cell_number",  comp_flag, istatus)

  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(ec_sw_mean_wat(cell_index),kind=real4), &
                  "cld_emiss_sw_wat",&
                  DFNT_INT8,LINEAR_SCALING,min_ec,max_ec,"none",missing_value_real4, &
                  "grid_cell_number",  comp_flag, istatus)

  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(ec_sw_mean_ice(cell_index),kind=real4), &
                  "cld_emiss_sw_ice",&
                  DFNT_INT8,LINEAR_SCALING,min_ec,max_ec,"none",missing_value_real4, &
                  "grid_cell_number",  comp_flag, istatus)

  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(ec_sw_std(cell_index),kind=real4), &
                  "cld_emiss_sw_std",&
                  DFNT_INT8,LINEAR_SCALING,min_ec_std,max_ec_std,"none",missing_value_real4, &
                  "grid_cell_number",  comp_flag, istatus)

  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(ec_sw_std_wat(cell_index),kind=real4), &
                  "cld_emiss_sw_std_wat",&
                  DFNT_INT8,LINEAR_SCALING,min_ec_std,max_ec_std,"none",missing_value_real4, &
                  "grid_cell_number",  comp_flag, istatus)

  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(ec_sw_std_ice(cell_index),kind=real4), &
                  "cld_emiss_sw_std_ice",&
                  DFNT_INT8,LINEAR_SCALING,min_ec_std,max_ec_std,"none",missing_value_real4, &
                  "grid_cell_number",  comp_flag, istatus)

  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(Pc_sw_mean(cell_index),kind=real4), &
                  "cld_press_sw",&
                  DFNT_INT8,LINEAR_SCALING,min_pc,max_pc,"hPa",missing_value_real4, &
                  "grid_cell_number",  comp_flag, istatus)

  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(Pc_sw_mean_wat(cell_index),kind=real4), &
                  "cld_press_sw_wat",&
                  DFNT_INT8,LINEAR_SCALING,min_pc,max_pc,"hPa",missing_value_real4, &
                  "grid_cell_number",  comp_flag, istatus)

  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(Pc_sw_mean_ice(cell_index),kind=real4), &
                  "cld_press_sw_ice",&
                  DFNT_INT8,LINEAR_SCALING,min_pc,max_pc,"hPa",missing_value_real4, &
                  "grid_cell_number",  comp_flag, istatus)

  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(Zc_sw_mean(cell_index),kind=real4), &
                  "cld_height_sw",&
                  DFNT_INT8,LINEAR_SCALING,min_zc,max_zc,"km",missing_value_real4, &
                  "grid_cell_number",  comp_flag, istatus)

  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(Zc_sw_mean_wat(cell_index),kind=real4), &
                  "cld_height_sw_wat",&
                  DFNT_INT8,LINEAR_SCALING,min_zc,max_zc,"km",missing_value_real4, &
                  "grid_cell_number",  comp_flag, istatus)

  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(Zc_sw_mean_ice(cell_index),kind=real4), &
                  "cld_height_sw_ice",&
                  DFNT_INT8,LINEAR_SCALING,min_zc,max_zc,"km",missing_value_real4, &
                  "grid_cell_number",  comp_flag, istatus)

  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(beta_sw_mean(cell_index),kind=real4), &
                  "cld_beta_sw",&
                  DFNT_INT8,LINEAR_SCALING,min_beta,max_beta,"none",missing_value_real4, &
                  "grid_cell_number",  comp_flag, istatus)

  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(beta_sw_std(cell_index),kind=real4), &
                  "cld_beta_std_sw",&
                  DFNT_INT8,LINEAR_SCALING,min_beta_std,max_beta_std,"none",missing_value_real4, &
                  "grid_cell_number",  comp_flag, istatus)

  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(beta_sw_mean_wat(cell_index),kind=real4), &
                  "cld_beta_sw_wat",&
                  DFNT_INT8,LINEAR_SCALING,min_beta,max_beta,"none",missing_value_real4, &
                  "grid_cell_number",  comp_flag, istatus)

  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(beta_sw_mean_ice(cell_index),kind=real4), &
                  "cld_beta_sw_ice",&
                  DFNT_INT8,LINEAR_SCALING,min_beta,max_beta,"none",missing_value_real4, &
                  "grid_cell_number",  comp_flag, istatus)

  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(tau_sw_mean(cell_index),kind=real4), &
                  "cld_opd_sw",&
                  DFNT_INT8,LINEAR_SCALING,min_tau,max_tau,"none",missing_value_real4, &
                  "grid_cell_number",  comp_flag, istatus)

  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(tau_sw_mean_wat(cell_index),kind=real4), &
                  "cld_opd_sw_wat",&
                  DFNT_INT8,LINEAR_SCALING,min_tau,max_tau,"none",missing_value_real4, &
                  "grid_cell_number",  comp_flag, istatus)

  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(tau_sw_mean_ice(cell_index),kind=real4), &
                  "cld_opd_sw_ice",&
                  DFNT_INT8,LINEAR_SCALING,min_tau,max_tau,"none",missing_value_real4, &
                  "grid_cell_number",  comp_flag, istatus)

!-- vis
  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(tau_vis_mean(cell_index),kind=real4), &
                  "cld_opd_vis",&
                  DFNT_INT8,SQUARE_ROOT_SCALING,min_tau,max_tau,"none",missing_value_real4, &
                  "grid_cell_number",  comp_flag, istatus)

  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(tau_vis_mean_ice(cell_index),kind=real4), &
                  "cld_opd_vis_ice",&
                  DFNT_INT8,SQUARE_ROOT_SCALING,min_tau,max_tau,"none",missing_value_real4, &
                  "grid_cell_number",  comp_flag, istatus)

  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(tau_vis_mean_wat(cell_index),kind=real4), &
                  "cld_opd_vis_wat",&
                  DFNT_INT8,SQUARE_ROOT_SCALING,min_tau,max_tau,"none",missing_value_real4, &
                  "grid_cell_number",  comp_flag, istatus)

  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(tau_vis_std(cell_index),kind=real4), &
                  "cld_opd_std_vis",&
                  DFNT_INT8,SQUARE_ROOT_SCALING,min_tau_std,max_tau_std,"none",missing_value_real4, &
                  "grid_cell_number",  comp_flag, istatus)

  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(re_vis_mean(cell_index),kind=real4), &
                  "cld_reff_vis",&
                  DFNT_INT8,SQUARE_ROOT_SCALING,min_re,max_re,"microns",missing_value_real4, &
                  "grid_cell_number",  comp_flag, istatus)

  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(re_vis_mean_ice(cell_index),kind=real4), &
                  "cld_reff_vis_ice",&
                  DFNT_INT8,SQUARE_ROOT_SCALING,min_re,max_re,"microns",missing_value_real4, &
                  "grid_cell_number",  comp_flag, istatus)

  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(re_vis_mean_wat(cell_index),kind=real4), &
                  "cld_reff_vis_wat",&
                  DFNT_INT8,SQUARE_ROOT_SCALING,min_re,max_re,"microns",missing_value_real4, &
                  "grid_cell_number",  comp_flag, istatus)

  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(re_vis_std(cell_index),kind=real4), &
                  "cld_reff_std_vis",&
                  DFNT_INT8,SQUARE_ROOT_SCALING,min_re_std,max_re_std,"microns",missing_value_real4, &
                  "grid_cell_number",  comp_flag, istatus)

  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(lwp_vis_mean(cell_index),kind=real4), &
                  "lwp_vis",&
                  DFNT_INT8,SQUARE_ROOT_SCALING,min_lwp,max_lwp,"g/cm^2",missing_value_real4, &
                  "grid_cell_number",  comp_flag, istatus)

  call WRITE_CLAVRX_HDF4_SDS(sd_id_cell,real(iwp_vis_mean(cell_index),kind=real4), &
                  "iwp_vis",&
                  DFNT_INT8,SQUARE_ROOT_SCALING,min_iwp,max_iwp,"g/cm^2",missing_value_real4, &
                  "grid_cell_number",  comp_flag, istatus)


!-----------------------------------------------------------------------
!--- close file
!------------------------------------------------------------------------
   istatus = sfend(sd_id_cell)

!--- deallocate memory
 deallocate(cell_index)

end subroutine WRITE_GRIDCELL_DATA



 !------------------------------------------------------
 ! end of module
 !------------------------------------------------------

end module CELL_HDF_ROUTINES_1
