!$Id: scaling_params.f90,v 1.14 2010/09/23 15:48:43 wstraka Exp $
module SCALING_PARAMETERS
!----------------------------------------------------------------------------
!@***h* PATMOSX/scaling_params.f90
! 
! NAME
!   scaling_params.f90
! 
! LOCATION
!   /patmosx/src/scaling_params.f90
! 
! PURPOSE
!   Contains the scaling parameters for all scaling/unscaling performed
!   in CLAVR-x
! 
! DESCRIPTION
!   Contains the scaling parameters for all scaling/unscaling performed
!   in CLAVR-x
!
! AUTHOR
!   Andrew Heidinger, NOAA/NESDIS
! 
! CREATION DATE
!   unk
!
! COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
!   THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
!   TO USERS.
!
! NOTES
!   none
!
!@*****
!------------------------------------------------------------------------
!
! MODULE name: SCALING_PARAMETERS
! 
! Module contains the scaling parameters for all scaling/unscaling performed
!       in CLAVR-x
!----------------------------------------------------------------------------

 use CONSTANTS
 implicit none

 integer(kind=int4), parameter, public:: one_byte_max = 127, &   !(2**8)/2 -1
                                          one_byte_min = -127     !-(2**8)/2
 integer(kind=int4), parameter, public:: two_byte_max = 32767, & !(2**15)/2 - 1
                                          two_byte_min = -32767   !-(2**15)/2
  
 real, parameter, public:: min_alb1 = -2.0, max_alb1 = 120.0
 real, parameter, public:: min_alb2 = -2.0, max_alb2 = 120.0
 real, parameter, public:: min_alb3a = -2.0, max_alb3a = 120.0
 real, parameter, public:: min_alb3b = -20.0, max_alb3b = 80.0
 real, parameter, public:: min_rad3b = 0.0, max_rad3b = 4.0
 real, parameter, public:: min_rad4 = 0.0, max_rad4 = 200.0
 real, parameter, public:: min_rad5 = 0.0, max_rad5 = 200.0
 real, parameter, public:: min_t3b = 180.0, max_t3b = 340.0
 real, parameter, public:: min_t4 = 180.0, max_t4 = 340.0
 real, parameter, public:: min_t5 = 180.0, max_t5 = 340.0
 real, parameter, public:: min_t45 = -4.0, max_t45 = 12.0
 real, parameter, public:: min_btd_3b_4 = -20.0, max_btd_3b_4 = 100.0
 real, parameter, public:: min_alb1_std = 0.0, max_alb1_std = 100.0
 real, parameter, public:: min_alb2_std = 0.0, max_alb2_std = 100.0
 real, parameter, public:: min_alb3a_std = 0.0, max_alb3a_std = 100.0
 real, parameter, public:: min_alb3b_std = 0.0, max_alb3b_std = 100.0
 real, parameter, public:: min_rad3b_std = 0.0, max_rad3b_std = 4.0
 real, parameter, public:: min_rad4_std = 0.0, max_rad4_std = 20.0
 real, parameter, public:: min_rad5_std = 0.0, max_rad5_std = 20.0
 real, parameter, public:: min_t3b_std = 0.0, max_t3b_std = 20.0
 real, parameter, public:: min_t4_std = 0.0, max_t4_std = 20.0
 real, parameter, public:: min_t5_std = 0.0, max_t5_std = 20.0
 real, parameter, public:: min_t45_std = 0.0, max_t45_std = 12.0
 real, parameter, public:: min_tsfc = 220.0, max_tsfc = 340.0, min_tsfc_std = 0.0, max_tsfc_std = 20.0
 real, parameter, public:: min_sst = 265.0, max_sst = 310.0, min_sst_std = 0.0, max_sst_std = 20.0
 real, parameter, public:: min_lst = 220.0, max_lst = 340.0, min_lst_std = 0.0, max_lst_std = 20.0
 real, parameter, public:: min_ist = 200.0, max_ist = 300.0, min_ist_std = 0.0, max_ist_std = 20.0
 real, parameter, public:: min_ndvi = -0.5, max_ndvi = 1.0, min_ndvi_std = 0.0, max_ndvi_std = 1.0
 real, parameter, public:: min_ang = 0.0, max_ang = 180.0
 real, parameter, public:: min_solaz = 0.0, max_solaz = 360.0
 real, parameter, public:: min_z = -500, max_z = 10000
 real, parameter, public:: min_lat = -90.0, max_lat = 90.0
 real, parameter, public:: min_lon = -180, max_lon = 180.0
 real, parameter, public:: min_zc = 0.0, max_zc = 20.0
 real, parameter, public:: min_pc = 0.0, max_pc = 1100.0
 real, parameter, public:: min_tc = 180.0, max_tc = 320.0, min_tc_std = 0.0, max_tc_std = 40.0
 real, parameter, public:: min_tc_error = 0.0, max_tc_error = 100.0
 real, parameter, public:: min_ec = 0.0, max_ec = 1.0, min_ec_std = 0.0, max_ec_std = 1.0
 real, parameter, public:: min_beta = 0.0, max_beta = 2.0, min_beta_std = 0.0, max_beta_std = 2.0
 real, parameter, public:: min_tau = -0.2, max_tau = 160.0, min_tau_std = 0.0, max_tau_std = 100.0
 real, parameter, public:: min_re = 0.0, max_re = 160.0, min_re_std = 0.0, max_re_std = 100.0
 real, parameter, public:: min_lwp = 0.0, max_lwp = 2000.0, min_lwp_std = 0.0, max_lwp_std = 100.0
 real, parameter, public:: min_iwp = 0.0, max_iwp = 2000.0, min_iwp_std = 0.0, max_iwp_std = 100.0
 real, parameter, public:: min_frac = 0.0, max_frac = 1.0
 real, parameter, public:: min_tpw = 0.0, max_tpw = 10.0
 real, parameter, public:: min_rh = 0.0, max_rh = 110.0
 real, parameter, public:: min_hght500 = 4500.0, max_hght500 = 6500.0
 real, parameter, public:: min_aot = -0.2, max_aot = 5.0, min_aot_std = 0.0, max_aot_std = 1.0
 real, parameter, public:: min_olr = 50.0, max_olr = 350.0, min_olr_std = 0.0, max_olr_std = 100.0
 real, parameter, public:: min_swar = 0.0, max_swar = 500.0, min_swar_std = 0.0, max_swar_std = 100.0
 real, parameter, public:: min_weasd = 0.0, max_weasd = 5500.0
 real, parameter, public:: min_wndspd = 0.0, max_wndspd = 50.0 
 real, parameter, public:: min_wnddir = 0.0, max_wnddir = 360.0 
 real, parameter, public:: min_ems3b = 0.5, max_ems3b = 3.0 
 real, parameter, public:: min_etropo = -0.5, max_etropo = 1.2 
 real, parameter, public:: min_ttrop = 160.0, max_ttrop = 260
 real, parameter, public:: min_rsr = -2.0, max_rsr = 10.0, min_rsr_std = 0.0, max_rsr_std = 2.0
 real, parameter, public:: min_ash_mass = 0.0, max_ash_mass = 100.0, min_ash_mass_std = 0.0, max_ash_mass_std = 100.0   !ton/km^2
 real, parameter, public:: min_albedo = 0.0, max_albedo = 1.0, min_transmission = 0.0, max_transmission = 1.0 
 real, parameter, public:: min_dcomp_atmos_vis = 0.0,max_dcomp_atmos_vis = 1.5, &
                           min_dcomp_atmos_ir = 0.0, max_dcomp_atmos_ir = 1.5 
 real, parameter, public:: min_sfc_ems = 0.75, max_sfc_ems = 1.0 

end module SCALING_PARAMETERS
